unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ADF_file_routines, ADF_diff_funcs, IdGlobal, IniFiles;


const
  MAX_ALTS_READ = 2;



type
  TForm1 = class(TForm)
    memo: TMemo;
    edtMaster: TEdit;
    edtAlt: TEdit;
    OpenDialog: TOpenDialog;
    Button1: TButton;
    Button2: TButton;
    btnProcess: TButton;
    edtDifference: TEdit;
    btnSaveDebug: TButton;
    SaveDialog: TSaveDialog;
    lblMasterProgress: TLabel;
    btnProcessed: TButton;
    edtProcessed: TEdit;
    edtNumAltsToProcess: TEdit;
    Label2: TLabel;
    Label1: TLabel;
    lblAltProgress: TLabel;
    Button3: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure btnProcessClick(Sender: TObject);
    procedure FileSearch(theList :TStringList; const PathName, FileName : string; const InDir : boolean);
    function compareFiles(master, alt : TADFlib; threshold :LongInt) :real;
    procedure btnSaveDebugClick(Sender: TObject);
    procedure btnProcessedClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private
    { Private declarations }
    fastabort   :Boolean;

    function validateInput    :boolean;
  public
    { Public declarations }



  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
begin
  if (OpenDialog.Execute) then
    edtMaster.Text := ExtractFilePath(OpenDialog.FileName);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  if (OpenDialog.Execute) then
    edtAlt.Text := ExtractFilePath(OpenDialog.FileName);
end;

procedure TForm1.btnProcessClick(Sender: TObject);
var

  percentdiff     :real;
  threshold       :LongInt;

  altsProcessed   :Integer;
  altsToProcessInThisBatch   :Integer;
  altcnt          :Integer;
  i, j, l, n      :Integer;

  move_ok         :LongBool;

  masFileName     :String;
  newFileName     :String;

  masterDirList   :TStringList;      // Directory list of master files.
  altDirList      :TStringList;      // Directory list of alt files to process.

  altFileList     :TStringList;

  masFile         :TADFlib;
  altFile         :TADFlib;

begin

  //============================================================================
  // VALIDATE INPUT ON FORM AND INIT VARIABLES
  //============================================================================
  if (not validateInput) then
  begin
    btnProcess.Enabled := true;
    exit;
  end;

  Memo.Lines.Clear;
  btnProcess.Enabled := false;
  fastabort := false;

  //============================================================================
  // CONVERT THE PERCENTAGE THRESHOLD ENTERED ON FROM INTO NUMBER OF BYTES
  //============================================================================
  percentdiff := (StrToInt(edtDifference.text) / 100) * 901120;
  threshold   := Round(percentdiff);

  //============================================================================
  // GENERATE LIST OF MASTER FILES
  //============================================================================
  masterDirList := TStringList.create;
  FileSearch(masterDirList, edtMaster.Text, '*.*', true);

  n := masterDirList.count-1;
  while (n >=0) do
  begin
    masFileName := masterDirList.Strings[n];

    if ((Pos('[a', masFileName) <> 0)
    or  (Pos('[u', masFileName) <> 0)
    or ((Pos('.zip', masFileName) = 0) and (Pos('.adf', masFileName) = 0))) then
    begin
      // Strip out alt and underdumped files from the master list
      // Also ignore files that dont have extension .zip or .adf
      masterDirList.Delete(n);
    end;
    Dec(n);
  end;

  //============================================================================
  // GENERATE LIST OF ALT FILES
  //============================================================================
  altDirList := TStringList.Create;
  FileSearch(altDirList, edtAlt.Text, '*.*', true);

  n := altDirList.count-1;
  while (n >=0) do
  begin
    newFileName := altDirList.Strings[n];

    if ((Pos('.zip', newFileName) = 0) and (Pos('.adf', newFileName) = 0)) then
    begin
      // Ignore files that dont have extension .zip or .adf
      altDirList.Delete(n);
    end;
    Dec(n);
  end;

  //============================================================================
  // DEBUG
  //============================================================================
  Memo.Lines.Add('Files to process in master dir : ' + IntToStr(masterDirList.Count));
  Memo.Lines.Add('Files to process in alt dir:     ' + IntToStr(altDirList.Count));
  Memo.Lines.Add('');

  //============================================================================
  // MAIN PROCESSING STARTS HERE....
  //============================================================================
  altcnt        := 0;
  altsProcessed := 0;
  altsToProcessInThisBatch := StrToInt(edtNumAltsToProcess.Text);

  while (altsProcessed < altDirList.count) do
  begin
    // Create the altFileList object
    altFileList := TStringList.Create;

    // Inner loop so we load a set number of alt files all in one go.
    // Making sure we dont go off the end
    n := 0;
    while ((n < altsToProcessInThisBatch) and (altsProcessed < altDirList.count)) do
    begin
      // Load alt file.
      altFile := TADFlib.Create;
      altFile.loadADF(altDirList[altsProcessed], false);

      // Add the new altFile to the altFileList

      if altFile.FileLoaded then
      begin
        altFileList.AddObject(altDirList[altsProcessed], altFile);
        Memo.lines.add('Loaded alt file: ' + altDirList[altsProcessed]);
        Inc(altsProcessed);
        Inc(n);
      end
      else
      begin
        Memo.lines.add('Error: bad file size. File Ignored:  ' + altDirList[altsProcessed] + ' Size = ' + IntToStr(altFile.floppySize));
        altDirList.Delete(altsProcessed);
      end;
    end;

    // At this point we have n alt files loaded.
    // Debug. Step through the loaded files and display each files CRC.
    {
    for i := 0 to n-1 do
    begin
      altFile := altFileList.Objects[i] as TADFlib;
      Memo.Lines.add(Format('%8.2x', [altFile.CRC32]));

      // Destroy the ADF object
      altFileList.Objects[i] := nil;
    end;
    }

    // We now have to loop through all the 'master' files and compare each
    // master file against all the files in our current altFileList
    // If an alt file is similar to a master file then rename the file
    // and move it away to another folder.

    for i := 0 to masterDirList.Count-1 do
    begin
      // Load the master file.
      masFile := TADFlib.Create;
      masFile.loadADF(masterDirList.Strings[i], false);

      lblMasterProgress.Caption := 'Loaded master file: ' + ExtractFileName(masterDirList.Strings[i]);

      if not masFile.FileLoaded then
      begin
        FreeAndNil(masFile);
        continue;
      end;


      // Compare each of the loaded alt files against the master file.
      // n := number of loaded alts

      j := 0;

      while (j < n) do
      begin

        lblAltProgress.Caption := 'Checking Alt: ' + ExtractFileName(altFileList.Strings[j]) + ' against Master.';

        Application.ProcessMessages;

        if (fastabort) then
        begin
          fastabort := false;
          btnProcess.Enabled := true;
          exit;
        end;

        altFile := altFileList.Objects[j] as TADFlib;

        // Check for exact dupe
        if (altFile.CRC32 = masFile.CRC32) then
        begin
          Memo.Lines.Add('    ' + altFile.getADFWindowsFileName + ' is exact dupe of:' + ExtractFileName(masterDirList.Strings[i]));

          // Move the dupe to the 'Processed' directory
          // If file is a zip, dont bother renaming the file within the zip as
          // this dupe will be deleted.
          l := length(altFile.getADFWindowsFileName);
          masFileName := ExtractFileName(masterDirList.Strings[i]);
          newFileName := edtProcessed.Text + '[DUPE] ' +
                          copy(masFileName, 0, length(masFileName)-4) +
                          copy(altFile.getADFWindowsFileName, l-3, l);

          // If file already exists in 'Processed' directory then delete file in alts
          // directory otherwise rename and move the alt file.
          if FileExists(newFileName) then
            DeleteFile(altFileList.Strings[j])
          else
            move_ok := MoveFile(PChar(altFileList.Strings[j]), PChar(newFileName));


          altFile := nil;
          altFileList.Objects[j] := nil;
          altFileList.Delete(j);
          Dec(n);   // We have one less alt file loaded.
          continue; // Dont need to increment j counter.
        end;

        // If got to here then the alt file is not an exact match
        percentdiff := compareFiles(masFile, altFile, threshold);

        if (percentdiff <= StrToInt(edtDifference.text)) then
        begin
          // If alt file is within the percentage difference then rename the file.
          newFileName := ExtractFileName(masterDirList.Strings[i]);
          newFileName := copy(newFileName, 0, length(newFileName)-4) +
                          '[a' + Format('%8x', [altFile.CRC32]) + ']' +
                          '[diff ' + Format('%2f', [percentdiff]) + ']';

          if (altFile.zipFile.Count <> 0) then
          begin
            // File is a Zip file - Rename the file within the zip
            altFile.zipFile.Name[0] := newFileName + '.adf';
            altFile.zipFile.SaveToFile(edtAlt.Text + altFile.getADFWindowsFileName);
            newFileName := newFileName + '.zip';  // File is zip file
          end
          else
          begin
            newFileName := newFileName + '.adf';  // File not a zip file
          end;


          // If file already exists in 'Processed' directory then delete file in alts
          // directory otherwise rename and move the alt file.
          if FileExists(edtProcessed.Text + newFileName) then
            DeleteFile(altFileList.Strings[j])
          else
            move_ok := MoveFile(PChar(altFileList.Strings[j]), PChar(edtProcessed.Text + newFileName));

          Memo.lines.add('    Alt file: ' + edtAlt.Text + altFile.getADFWindowsFileName +
                         ' Renamed To: ' + edtProcessed.Text + newFileName);

          Inc(altcnt);
          altFile := nil;
          altFileList.Objects[j] := nil;
          altFileList.Delete(j);
          Dec(n);   // We have one less alt file loaded.
          continue; // Dont need to increment j counter.

        end;

        Inc(j);
      end;
      FreeAndNil(masFile);
    end;

    // Destroy the altFileList object objects
    for i := 0 to altFileList.Count-1 do
      altFileList.Objects[i] := nil;

    FreeAndNil(altFileList);

    Memo.lines.add('');
    
  end;  // main while loop

  FreeAndNil(masterDirList);
  FreeAndNil(altDirList);
  btnProcess.Enabled := true;
  showMessage('Finished!');

end;

{===============================================================================
 FUNCTION:      FileSearch
 DESCRIPTION:   Builds a list of files in a given directory
===============================================================================}
procedure TForm1.FileSearch(theList :TStringList; const PathName, FileName : string; const InDir : boolean);
var Rec  : TSearchRec;
    Path : string;
    tmp   :Integer;
    
begin
  Path := IncludeTrailingPathDelimiter(PathName);
  if FindFirst(Path + FileName, faAnyFile - faDirectory, Rec) = 0 then
  try
    repeat
      theList.Add(Path + Rec.Name);
    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;

  If not InDir then Exit;

  if FindFirst(Path + '*.*', faDirectory, Rec) = 0 then
  try
    repeat
      tmp := (Rec.Attr and faDirectory);

      if tmp > 0 then
      begin
        if (Rec.Name <> '.') and (Rec.Name <> '..') then
          FileSearch(theList, Path + Rec.Name, FileName, True);
      end;

    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;
end; //procedure FileSearch


{===============================================================================
 FUNCTION:      CompareFiles
 DESCRIPTION:   Builds a list of files in a given directory
 ACCEPTS:       master file, alt file, threshold
===============================================================================}
function TForm1.compareFiles(master, alt : TADFlib; threshold :LongInt): real;
var
  left_b    :byte;
  right_b   :byte;
  cnt       :integer;
  offset    :longint;  // offset value of first byte that is different.
  nbytes    :longint;  // cnt of number of subsequent bytes that are different
  totnbytes :longint;

begin

  cnt      := 0;
  offset    := -1;
  nbytes    := 0;
  totnbytes := 0;

  while (cnt < 901120) do
  begin
    left_b  := master.getByteFromDiskMap(cnt);
    right_b := alt.getByteFromDiskMap(cnt);

    if ((left_b <> right_b) and (offset = -1)) then
      offset := cnt;

    if (left_b <> right_b) then
      nbytes := nbytes + 1;

    if (offset >= 0) and (left_b = right_b) then
    begin
      offset := -1;
      totnbytes := totnbytes + nbytes;

      // Exit early if we go over the threshold value
      if (totnbytes > threshold) then
      begin
        Result := 200;
        exit;
      end;

      nbytes := 0;
    end;
    Inc(cnt);
  end;

  if (offset >= 0) then
  begin
    // We havent processed last block of different bytes!
    totnbytes := totnbytes + nbytes;
  end;

  Result := (totnbytes / 901120) * 100;
  //Memo.Lines.add('Alt file is ' + Format('%2f', [pcent]) + '% different to master');
end;



procedure TForm1.btnSaveDebugClick(Sender: TObject);
begin
  if (SaveDialog.Execute) then
    memo.Lines.SaveToFile(SaveDialog.FileName);
end;

procedure TForm1.btnProcessedClick(Sender: TObject);
begin
  if (OpenDialog.Execute) then
    edtProcessed.Text := ExtractFilePath(OpenDialog.FileName);
end;

function TForm1.validateInput: boolean;
var
  n   :Integer;
  
begin

  Result := true;

  if ((length(edtMaster.Text) = 0)
  or  (length(edtAlt.Text) = 0)) then
  begin
    showMessage('Please make sure the master and alt directories are set.');
    Result := false;
  end;

  try
    n := StrToInt(edtDifference.Text)
  except
    on EConvertError do
    begin
      showMessage('Percentage is not a numeric value');
      Result := false;
    end;
  end;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  fastabort := true;
end;

procedure TForm1.FormShow(Sender: TObject);
var
  ini   :TIniFile;
begin

  if FileExists('AltFinder.ini') then
  begin
    ini := TIniFile.Create('./AltFinder.ini');
    edtMaster.Text := ini.ReadString('DIR_LOCATIONS', 'MASTER', '');
    edtAlt.Text := ini.ReadString('DIR_LOCATIONS', 'ALT', '');
    edtProcessed.Text := ini.ReadString('DIR_LOCATIONS', 'PROCESSED', '');
    FreeAndNil(ini);
  end;
end;

end.
