unit DragDrop;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, shellapi, ExtCtrls, Grids, ComCtrls, Menus, ADF_File_Routines;

type
  TForm1 = class(TForm)
    doc: TRichEdit;
    Button1: TButton;
    SaveDialog1: TSaveDialog;
    Button2: TButton;
    Panel1: TPanel;
    Label1: TLabel;
    cbDiskLabel: TCheckBox;
    cbDirList: TCheckBox;
    cbNDOS: TCheckBox;
    cbBootable: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender:  TObject; var Action: TCloseAction);
    procedure AppMessage(var Msg: Tmsg; var Handled: Boolean);
    procedure processFile(fname: String);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure cbNDOSClick(Sender: TObject);
    procedure cbDiskLabelClick(Sender: TObject);
    procedure cbDirListClick(Sender: TObject);

  private
    { Private declarations }
    currADF    :TADFlib;

  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(Form1.Handle, true);
  DragAcceptFiles(Panel1.Handle, true);
  DragAcceptFiles(doc.Handle, true);
  Application.OnMessage := AppMessage;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  currADF := TADFlib.Create;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  FreeAndNil(currADF);
end;

procedure TForm1.AppMessage(var Msg: Tmsg; var Handled: Boolean);
const
  BufferLength      :Word = 255;
var
  DroppedFilename   :string;
  FileIndex         :longword;
  QtyDroppedFiles   :word;
  pDroppedFilename  :array [0..255] of Char;
  DroppedFileLength :word;

begin
  FileIndex := $FFFFFFFF;
  QtyDroppedFiles := DragQueryFile(Msg.WParam, FileIndex, pDroppedFilename, BufferLength);

  if Msg.Message = WM_DROPFILES then
  begin
    for FileIndex := 0 to (QtyDroppedFiles - 1) do
    begin
      DroppedFileLength := DragQueryFile(Msg.WParam, FileIndex, pDroppedFilename, BufferLength);
      DroppedFilename   := StrPas(pDroppedFilename);

      // Do something with dropped filename
      if (Msg.hwnd = Form1.Handle)
      or (Msg.hwnd = Panel1.Handle)
      or (Msg.hwnd = doc.Handle) then
        processFile(DroppedFileName);
    end;

    DragFinish(Msg.WParam);
    Handled := true;
  end;
end;



procedure TForm1.processFile(fname  :String);
var
  i   :Integer;
  j   :Integer;
  f   :TDirectoryEntry;
  l   :string;

begin

  currADF.loadADF(fname);   // Load the disk image


  if (currADF.block.getBlockSecType(880) <> 'Root') then
  begin
    doc.Lines.Add('');
    doc.Lines.Add(fname);
    doc.Lines.Add('');
    doc.Lines.Add('        Not a standard AmigaDOS disk');

    if cbBootable.Checked then
    begin
      // Is disk bootable?
      if (currADF.isBootable) then
        doc.Lines.Add('Bootable  : YES')
      else
        doc.Lines.Add('Bootable  : NO');

      doc.Lines.add('');
    end;

  end
  else
  begin

    if not cbNDOS.Checked then
    begin

      doc.Lines.Add('');

      with doc do
      begin
        SelStart := Length(doc.Text);  // at end of rich edit
        SelAttributes.Color := clBlue;
        SelAttributes.Size  := 10;
        SelText := fname;

        if (currADF.NameOfFileInZip <> '') then
          SelText := SelText + '    (' + currADF.NameOfFileInZip + ')';

        SelText := SelText + #13#10;

        // SelStart is now at the end of the inserted text, so for the next text
        SelStart := Length(doc.Text);
        SelAttributes.Color := clBlack;
        SelAttributes.Style := [ ]; // an empty set
        SelAttributes.Size := 8;
        SelText := ' ';
      end;

      if cbDiskLabel.Checked then
      begin
        doc.Lines.Add('');
        doc.Lines.Add('');
        doc.Lines.Add('Disk Label: ' + currADF.getDiskLabel);
      end;


      if cbBootable.Checked then
      begin
        // Is disk bootable?
        if (currADF.isBootable) then
          doc.Lines.Add('Bootable  : YES')
        else
          doc.Lines.Add('Bootable  : NO');

        doc.Lines.add('');
      end;

      if cbDirList.Checked then
      begin
        doc.Lines.Add('');
        for i := 0 to currADF.dirList.Count-1 do
        begin
          f := TDirectoryEntry(currADF.dirList.Objects[i]);

          l := '';

          for j := 0 to f.level-1 do
            l := l + '    ';

          if (f.parentdir_name = 'ROOT') then
          begin
            l := l + f.name;
          end
          else
            l := l + f.parentdir_name + '/' + f.name;

          doc.Lines.add(l)
        end;
      end;
    end;
  end;

  currADF.unLoadADF;

end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  if (SaveDiaLog1.Execute) then
  begin
    doc.Lines.SaveToFile(SaveDiaLog1.FileName);
    showMessage('File saved as ' + SaveDiaLog1.FileName);
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  doc.Lines.Clear;
end;

procedure TForm1.cbNDOSClick(Sender: TObject);
begin
  if cbNDOS.Checked then
  begin
    cbDiskLabel.Checked := false;
    cbDirList.Checked   := false;
  end;
end;

procedure TForm1.cbDiskLabelClick(Sender: TObject);
begin
  if cbDiskLabel.checked then
    cbNDOS.checked := false;
end;

procedure TForm1.cbDirListClick(Sender: TObject);
begin
  if cbDirList.checked then
    cbNDOS.Checked := false;
end;

end.

