unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ADF_file_routines, CompareRoutines;

type
  TfrMain = class(TForm)
    edtSourceDir    :TEdit;
    edtProcessedDir :TEdit;
    edtDupeDir      :TEdit;

    btnSourceDir    :TButton;
    btnProcessedDir :TButton;
    btnDupeDir      :TButton;

    lblSource       :TLabel;
    lblProcessed    :TLabel;
    lblDupe         :TLabel;

    btnCheckDupes: TButton;

    Memo1: TMemo;
    btnSave: TButton;
    dlgDirectory: TOpenDialog;
    btnStop: TButton;

    procedure btnCheckDupesClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnProcessedDirClick(Sender: TObject);
    procedure btnSourceDirClick(Sender: TObject);
    procedure btnDupeDirClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private

    fileList  :TStringList;
    fastabort :Boolean;

    procedure FileSearch(theList :TStringList; const PathName, FileName : string; const InDir : boolean);




  public
    { Public declarations }
  end;

var
  frMain: TfrMain;

implementation
{$R *.DFM}
uses uFileInfo;


procedure TfrMain.FileSearch(theList :TStringList; const PathName, FileName : string; const InDir : boolean);
var Rec  : TSearchRec;
    Path : string;
    tmp   :Integer;
    
begin
  Path := IncludeTrailingPathDelimiter(PathName);
  if FindFirst(Path + FileName, faAnyFile - faDirectory, Rec) = 0 then
  try
    repeat
      theList.Add(Path + Rec.Name);
    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;

  If not InDir then Exit;

  if FindFirst(Path + '*.*', faDirectory, Rec) = 0 then
  try
    repeat
      tmp := (Rec.Attr and faDirectory);

      if tmp > 0 then
      begin
        if (Rec.Name <> '.') and (Rec.Name <> '..') then
          FileSearch(theList, Path + Rec.Name, FileName, True);
      end;

    until FindNext(Rec) <> 0;
  finally
    FindClose(Rec);
  end;
end; //procedure FileSearch




procedure TfrMain.btnCheckDupesClick(Sender: TObject);
var
  same      :Boolean;
  lb_ok     :Boolean;
  i,j       :Integer;
  numfiles  :Integer;
  fname     :string;
  dupefname :string;


  master    :TADFlib;
  slave     :TADFlib;

begin

  if ((length(edtSourceDir.Text) = 0)
  or  (length(edtProcessedDir.Text) = 0)
  or  (length(edtDupeDir.Text) = 0)) then
    showMessage('Please make sure all directories are selected.')
  else
  begin

    Memo1.Lines.Clear;
    fastabort := false;
    btnStop.Enabled := true;

    fileList := TStringList.create;
    FileSearch(fileList, edtSourceDir.Text, '*.*', true);
    numfiles := fileList.Count;

    Memo1.Lines.Add('Total files found : ' + IntToStr(numfiles));

    // Go through all files in the 'to process' list backwards
    // as we remove the file from the list after processing.

    master := TADFlib.Create;
    slave  := TADFlib.Create;


    while (fileList.Count-1 >= 0) do
    begin

      i := fileList.count-1;
      fname := ExtractFileName(fileList.Strings[i]);

      // Load the main file to check against.
      master.loadADF(fileList.Strings[i]);

      for j := fileList.count-2 downto 0 do
      begin

        Application.ProcessMessages;
        if (fastabort) then
          break;

        //Memo1.lines.add('Comparing '+fileList.Strings[i]+' to '+fileList.Strings[j]);

        // Compare a tiny bit of the slave files with the master file first to see if we need
        // to check the whole thing.

        slave.loadADF(fileList.Strings[j]);

        if master.CRC32 = slave.CRC32 then
          same := true
        else
          same := false;

        slave.unLoadADF;

        if same then
        begin
          dupefname := ExtractFileName(fileList.Strings[j]);
          Memo1.Lines.Add(fileList.Strings[j]+' is a dupe of '+fileList.Strings[i]);
          lb_ok := MoveFile(PChar(fileList.Strings[j]), PChar(edtDupeDir.text+dupefname));
          fileList.Delete(j);
          i := fileList.Count-1;
        end;

      end;

      if (fastabort) then
        break;


      // Finally
      // Move the file we have just processed to the 'processed' dir
      lb_ok := MoveFile(PChar(fileList.Strings[i]), PChar(edtProcessedDir.text+fname));

      master.unLoadADF;
      fileList.Delete(i);
    end;


    btnStop.Enabled := false;
    master.Free;
    slave.Free;
    fileList.Free;
    fastabort := false;

    showMessage('Processing complete!');
  end;

end;


procedure TfrMain.btnSaveClick(Sender: TObject);
begin
  if (Memo1.Lines.Count > 0) then
  begin
    Memo1.Lines.SaveToFile('dupelog.txt');
    showMessage('File saved as dupelog.txt');
  end
  else
    showMessage('Nothing to save!');

end;



procedure TfrMain.btnSourceDirClick(Sender: TObject);
begin
  if (dlgDirectory.Execute) then
    edtSourceDir.Text := ExtractFilePath(dlgDirectory.FileName);
end;

procedure TfrMain.btnProcessedDirClick(Sender: TObject);
begin
  if (dlgDirectory.Execute) then
    edtProcessedDir.Text := ExtractFilePath(dlgDirectory.FileName);
end;


procedure TfrMain.btnDupeDirClick(Sender: TObject);
begin
  if (dlgDirectory.Execute) then
    edtDupeDir.Text := ExtractFilePath(dlgDirectory.FileName);
end;

procedure TfrMain.btnStopClick(Sender: TObject);
begin
  fastabort := true;
end;

end.
