unit DragDrop;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, shellapi, ExtCtrls, Grids, ComCtrls, myFrame, Menus,
  CompareRoutines;

type

  TfrmMain = class(TForm)
    MainMenu1: TMainMenu;
    btnCreateDiff: TButton;
    RightFrame: TADF;
    Memo1: TMemo;
    btnReadDiff: TButton;
    btnCalcCRC32: TButton;
    Button1: TButton;
    N1: TMenuItem;
    About1: TMenuItem;
    LeftFrame: TADF;
    chkExtra: TMenuItem;
    Button2: TButton;
    procedure AppMessage(var Msg: Tmsg; var Handled: Boolean);
    procedure FormCreate(Sender: TObject);

    procedure display_Data(Sender:  TObject);
    procedure LeftFrameViewClick(Sender: TObject);
    procedure RightFrameViewClick(Sender: TObject);
    procedure RightFrameAsciiViewTopLeftChanged(Sender: TObject);
    procedure btnCreateDiffClick(Sender: TObject);
    procedure btnReadDiffClick(Sender: TObject);
    procedure btnCalcCRC32Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure LeftFramebtnSaveADFClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure chkExtraClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);

  private
    { Private declarations }

  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  ADF_diff_funcs, CRC32, aboutForm;

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(LeftFrame.TabSheet1.Handle, true);
  DragAcceptFiles(RightFrame.TabSheet1.Handle, true);

  DragAcceptFiles(LeftFrame.edtFileToInsert.Handle, true);
  DragAcceptFiles(RightFrame.edtFileToInsert.Handle, true);

  Application.OnMessage := AppMessage;
end;


procedure TfrmMain.AppMessage(var Msg: Tmsg; var Handled: Boolean);
begin
  if Msg.Message = WM_DROPFILES then
  begin

    if Msg.hwnd = LeftFrame.Tabsheet1.Handle then
    begin
      LeftFrame.Load(Msg);
      display_data(LeftFrame);
    end;

    if Msg.hwnd = RightFrame.Tabsheet1.Handle then
    begin
      RightFrame.Load(Msg);
      display_data(RightFrame);
    end;

    if Msg.hwnd = LeftFrame.edtFileToInsert.Handle then
    begin
      LeftFrame.Load(Msg);
    end;

    if Msg.hwnd = RightFrame.edtFileToInsert.Handle then
    begin
      RightFrame.Load(Msg);
    end;

    DragFinish(Msg.WParam);
    Handled := true;
  end;
end;


procedure TfrmMain.display_Data(Sender: TObject);
begin

  if (Sender = LeftFrame) then
  begin
    // Display the contents of the sector
    LeftFrame.displayRawData('HEX', 0, LeftFrame.HexView, 'NORMAL', nil);
    LeftFrame.displayRawData('ASCII', 0, LeftFrame.AsciiView, 'NORMAL', nil);
  end
  else
  begin
    // Display the contents of the sector
    RightFrame.displayRawData('HEX', 0, RightFrame.HexView, 'COMPARE', LeftFrame);
    RightFrame.displayRawData('ASCII', 0, RightFrame.AsciiView, 'COMPARE', LeftFrame);
  end;

end;

procedure TfrmMain.LeftFrameViewClick(Sender: TObject);
begin
  // This also gets called by the AsciiViewClick too.
  RightFrame.m_col_clicked := LeftFrame.m_col_clicked;
  RightFrame.m_row_clicked := LeftFrame.m_row_clicked;
  RightFrame.HexView.Selection := LeftFrame.HexView.Selection;
end;

procedure TfrmMain.RightFrameViewClick(Sender: TObject);
begin
  // This also gets called by the AsciiViewClick too.
  LeftFrame.m_col_clicked := RightFrame.m_col_clicked;
  LeftFrame.m_row_clicked := RightFrame.m_row_clicked;
  LeftFrame.HexView.Selection := RightFrame.HexView.Selection;
end;

procedure TfrmMain.RightFrameAsciiViewTopLeftChanged(Sender: TObject);
begin
  RightFrame.AsciiViewTopLeftChanged(Sender);
  LeftFrame.AsciiView.TopRow := RightFrame.AsciiView.TopRow;
end;



procedure TfrmMain.btnCreateDiffClick(Sender: TObject);
var
  left_b  : byte;
  right_b : byte;
  i       : integer;
  j       : integer;
  n       : integer;
  cnt     : integer;
  offset  : longint;  // offset value of first byte that is different.
  nbytes  : longint;  // cnt of number of subsequent bytes that are different

  totnbytes   : longint;
  pcent   : real;

  crc     : longword;

  diffFileName : string;

  theFile : file;
  
  offsetPtr   : Pointer;
  entryList   : TEntryList;

begin

  if ((LeftFrame.ADF.FileLoaded <> true)
  or  (RightFrame.ADF.FileLoaded <> true)) then
  begin
    showMessage('You need to load a file into both Frames to perform this function.');
    exit;
  end;


  // Read through both files a byte at a time.
  // Offset = position of first byte of a sequence of bytes that are different
  // nbytes = number of bytes in the current sequence that are different.

  // This code creates a structure in memory as follows:
  //
  // EntryList (TStringList)
  //     TEntry object 1 =  '000001'
  //                            |------- offset value 1
  //                            |------- offset value 2
  //                            |------- etc...
  //     TEntry object 2 =  '000035'
  //     TEntry object 3 =  '000050'
  //     and so on...

  // Each TEntry record has its own TList of offset values. So by looking at the
  // name of the object ie: '000035' and then the list of offsets within that name
  // we can see that at starting at offset nn, the next 450 bytes of data are different.

  cnt     := 0;
  offset  := -1;
  nbytes  := 0;
  totnbytes := 0;
  entryList := TEntryList.Create;

  while (cnt < 901120) do
  begin
    left_b  := LeftFrame.ADF.getByteFromDiskMap(cnt);
    right_b := RightFrame.ADF.getByteFromDiskMap(cnt);

    if ((left_b <> right_b) and (offset = -1)) then
      offset := cnt;

    if (left_b <> right_b) then
      nbytes := nbytes + 1;

    if (offset > 0) and (left_b = right_b) then
    begin
      entryList.InsertEntry(nbytes, offset);
      //Memo1.lines.add('Spoint = ' + IntToStr(spoint) + ' - ' + ' nbytes = ' + IntToStr(nbytes));
      offset := -1;
      totnbytes := totnbytes + nbytes;
      nbytes := 0;
    end;
    Inc(cnt);
  end;

  if (offset > 0) then
  begin
    // We havent written last block of different bytes!
    entryList.InsertEntry(nbytes, offset);
    offset := -1;
    totnbytes := totnbytes + nbytes;
    nbytes := 0;
  end;

  entryList.sort;

  pcent := (totnbytes / 901120) * 100;
  Memo1.Lines.add('Alt file is ' + Format('%2f', [pcent]) + '% different to master');

  // DEBUG START
  {
  for i := 0 to entryList.count-1 do
  begin
    Memo1.Lines.add(entryList.Strings[i]);

    cnt := entryList.GetOffSetListCount(i);
    Memo1.Lines.Add(' This entry in the list has: ' + IntToStr(cnt) + ' offsets');

    for j := 0 to cnt-1 do
    begin
      Memo1.Lines.add('    ' + IntToStr(entryList.getOffsetItemValue(i, j)));
    end;
  end;
  }
  // DEBUG END


  // Write the sorted data to the diff file.

  // Diff file format:
  // First longword (4 bytes)  = CRC of master file
  // Second longword (4 bytes) = CRC of slave file

  // Next longword (4 bytes) = (Start of block) Number of bytes per offset that are different
  // Next longword (4 bytes) = Number of offsets within the current 'block'
  //     Next longword (4 bytes) = 1st offset value
  //     Next nnn bytes = the data to go at first offset
  //     Next longword (4 bytes) = 2nd offset value
  //     Next nnn bytes = the data to go at 2nd offset
  //   Repeat for all offsets within the 'block'
  diffFileName := RightFrame.edtFname.Text;
  diffFileName := copy(diffFileName, 1, length(diffFileName)-4) + '.diff';

  AssignFile(theFile, diffFileName);
  ReWrite(theFile, 1);
  Seek(theFile, 0);

  // First two longwords of the file are the master/slave adf CRC's
  crc := LeftFrame.ADF.CRC32;
  BlockWrite(theFile, crc, 4);
  crc := RightFrame.ADF.CRC32;

  BlockWrite(theFile, crc, 4);

  for i := 0 to entryList.count-1 do
  begin
    // Number of bytes in the sequence
    nbytes := StrToInt(entryList.Strings[i]);
    BlockWrite(theFile, nbytes , 4);

    // Number of offsets
    cnt := entryList.GetOffSetListCount(i);
    BlockWrite(theFile, cnt, 4);

    for j := 0 to cnt-1 do
    begin
      n := entryList.getOffsetItemValue(i, j);  // n = offsetvalue

      // Write the offset value
      BlockWrite(theFile, n, 4);

      offsetPtr := RightFrame.ADF.getDiskMapPtr2(n);
      BlockWrite(theFile, offsetPtr^, nbytes);
    end;
  end;

  CloseFile(theFile);

  Memo1.lines.add('File: ''' + diffFileName + ''' has been created.');


end;







procedure TfrmMain.btnReadDiffClick(Sender: TObject);
var
  n       : Longint;
  i     : Integer;
  s       : string;
  theFile : file;
  fpos    : longint;
  fsize   : longint;

  nBytes   : integer;
  nOffsets : integer;
  offset   : Longint;

  masterCRC : Longword;
  slaveCRC  : Longword;

  diffFileName  : string;

  theBytes : array[0..999] of byte;


begin

  if (RightFrame.ADF.FileLoaded <> true) then
  begin
    showMessage('You need to load a file into the Right Frame to perform this function.');
    exit;
  end;


  diffFileName := RightFrame.edtFname.Text;
  diffFileName := copy(diffFileName, 1, length(diffFileName)-4) + '.diff';

  Memo1.Lines.add('Analyzing diff file: ' + diffFileName);
  if not FileExists(diffFileName) then
  begin
    Memo1.Lines.add('diff file does not exist');
    exit;
  end;

  AssignFile(theFile, diffFileName);


  reset(theFile, 1);
  Seek(theFile, 0);

  fpos  := FilePos(theFile);
  fsize := FileSize(theFile);

  BlockRead(theFile, masterCRC, 4);
  BlockRead(theFile, slaveCRC,  4);

  Memo1.lines.add('Master file CRC = ' + IntToHex(masterCRC, 4));
  Memo1.lines.add('Slave  file CRC = ' + IntToHex(slaveCRC, 4));

  while (fpos < fsize) do
  begin
    BlockRead(theFile, nBytes, 4);
    BlockRead(theFile, nOffsets, 4);

    Memo1.Lines.add('Number of bytes in the sequence: ' + IntToStr(nBytes));
    Memo1.Lines.add('Number of Offsets = ' + IntToStr(nOffsets));

    for n := 0 to nOffsets-1 do
    begin
      BlockRead(theFile, offset, 4);
      BlockRead(theFile, theBytes, nBytes);

      fpos := FilePos(theFile);

      s := '';
      for i := 0 to nBytes-1 do
      begin
        s := s + ' ' + Format('%0.2x', [theBytes[i]]);
      end;
      s := 'Offset: ' + IntToStr(offset) + ' ' + s;
      Memo1.Lines.Add(s);
    end;
  end;

  CloseFile(theFile);
  Memo1.Lines.add('');
  
end;

procedure TfrmMain.btnCalcCRC32Click(Sender: TObject);
var
  r   : Longword;

begin
  r := LeftFrame.ADF.CRC32;
  Memo1.Lines.add('Left Frame  CRC32 = ' + IntToHex(r, 8));

  r := RightFrame.ADF.CRC32;
  Memo1.Lines.add('Right Frame CRC32 = ' + IntToHex(r, 8));

end;

procedure TfrmMain.Button1Click(Sender: TObject);
var
  p1, p2    : Pointer;

begin
  if ((LeftFrame.ADF.FileLoaded <> true)
  or  (RightFrame.ADF.FileLoaded <> true)) then
  begin
    showMessage('You need to load a file into both Frames to perform this function.');
    exit;
  end;
  
  p1 := LeftFrame.ADF.getDiskMapPtr(0);
  p2 := RightFrame.ADF.getDiskMapPtr(0);

  if (FastByteComp(p1, p2, 901120) = true) then
    Memo1.Lines.add('Both ADF images are identical')
  else
    Memo1.Lines.add('ADF images are different');

end;

procedure TfrmMain.LeftFramebtnSaveADFClick(Sender: TObject);
begin
  showMessage('Not implemented yet');
  //LeftFrame.btnSaveADFClick(Sender);

end;

procedure TfrmMain.About1Click(Sender: TObject);
var
  aboutFrm    :TfrmAbout;

begin

  aboutFrm := TfrmAbout.Create(Self);
  aboutFrm.ShowModal;
  FreeAndNil(aboutFrm);

end;

procedure TfrmMain.chkExtraClick(Sender: TObject);
begin
  chkExtra.Checked := not chkExtra.Checked;

  if (chkExtra.Checked) then
    frmMain.Height := 559
  else
    frmMain.Height := 401;



end;

procedure TfrmMain.Button2Click(Sender: TObject);
var
  n     :Integer;

begin
{
  if (LeftFrame.ADF.FileLoaded) then
  begin

    if (LeftFrame.ADF.zipFile.Count > 0) then
    begin
      // We have loaded a zip file
      Memo1.Lines.add('Files in zip = ' + IntToStr(LeftFrame.ADF.zipFile.Count));

      Memo1.Lines.add(LeftFrame.ADF.zipFile.Name[0] + ' has been renamed to: ' + 'renamed.adf');
      LeftFrame.ADF.zipFile.Name[0] := 'renamed.adf';

      Memo1.Lines.add(LeftFrame.ADF.zipFile.Name[1] + ' has been renamed to: ' + 'another.adf');
      LeftFrame.ADF.zipFile.Name[1] := 'another.adf';

      LeftFrame.ADF.zipFile.SaveToFile('D:\' + LeftFrame.ADF.getADFWindowsFileName);
    end
    else
    begin
      // We have loaded an ordinary file
      Memo1.Lines.add('No zip file loaded');
    end;
  end;
}

end;

end.

