unit InfoPage;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, ADF_File_Routines;

type

  {=============================================================================
   All references in this unit to ADF.block.xxxxx without checking to see if the
   ADF is loaded first is ok because this page cannot be accessed unless an ADF
   was loaded first.
  =============================================================================}
  TfrmInfomation = class(TForm)
    strgdFHBinfo: TStringGrid;
    Label1: TLabel;
    cbDataBlocks: TComboBox;
    strgdDBinfo: TStringGrid;
    filedataHexView: TDrawGrid;
    filedataAsciiView: TDrawGrid;

    procedure cbDataBlocksChange(Sender: TObject);
    procedure filedataHexViewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure filedataAsciiViewDrawCell(Sender: TObject; ACol,
      ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure filedataAsciiViewTopLeftChanged(Sender: TObject);
    procedure filedataHexViewTopLeftChanged(Sender: TObject);

  private
    { Private declarations }
      theADF  :TADFlib;
      fhb   :^TFileHeaderBlock;
      feb   :^TFileExtnBlock;

  public
    { Public declarations }
    procedure initInfoPage(adf  :TADFlib; blockNum :Integer);

    procedure dispFHBinfo(blockNum: Integer);   // Display file header block info
    procedure dispDBinfo(blockNum :Integer);    // Display data block info

  end;

var
  frmInfomation: TfrmInfomation;

implementation
uses myFrame, general_funcs;
{$R *.dfm}



procedure TfrmInfomation.initInfoPage(adf   :TADFlib; blockNum  :Integer);
begin
    theADF := adf;
    fhb := theADF.getFileHeaderBlockPtr(blockNum);
    dispFHBinfo(blockNum);

    // Force file details stringgrid to be populated
    cbDataBlocks.itemindex := 0;
    cbDataBlocks.OnChange(cbDataBlocks);

    // Finally, populate the 'whole data view'
    theADF.extractFile(blockNum, 'FileHeader');

end;



procedure TfrmInfomation.dispFHBinfo(blockNum :Integer);
var
  i               :Integer;
  extnBlockPtr    :Longword;

begin
  strgdFHBinfo.Cells[0,0]  := 'File Header Block';
  strgdFHBinfo.Cells[0,1]  := 'Primary Type';
  strgdFHBinfo.Cells[0,2]  := 'Header Key';
  strgdFHBinfo.Cells[0,3]  := 'High Seq';
  strgdFHBinfo.Cells[0,4]  := 'Data Size (unused)';
  strgdFHBinfo.Cells[0,5]  := 'First data block ptr';
  strgdFHBinfo.Cells[0,6]  := 'Checksum';
  strgdFHBinfo.Cells[0,7]  := 'Data Blocks';
  strgdFHBinfo.Cells[0,8]  := 'Unused1';
  strgdFHBinfo.Cells[0,9]  := 'User Id';
  strgdFHBinfo.Cells[0,10] := 'Group Id';
  strgdFHBinfo.Cells[0,11] := 'Protect Flags';
  strgdFHBinfo.Cells[0,12] := 'File Size';
  strgdFHBinfo.Cells[0,13] := 'Comment Length';
  strgdFHBinfo.Cells[0,14] := 'Comment';
  strgdFHBinfo.Cells[0,15] := 'Unused 2 (12 chars)';
  strgdFHBinfo.Cells[0,16] := 'Days';
  strgdFHBinfo.Cells[0,17] := 'Mins';
  strgdFHBinfo.Cells[0,18] := 'Ticks';
  strgdFHBinfo.Cells[0,19] := 'Filename length';
  strgdFHBinfo.Cells[0,20] := 'Filename';
  strgdFHBinfo.Cells[0,21] := 'Unused3 (1 char)';
  strgdFHBinfo.Cells[0,22] := 'Unused4 (1 longword)';
  strgdFHBinfo.Cells[0,23] := 'Real Entry';
  strgdFHBinfo.Cells[0,24] := 'Next link';
  strgdFHBinfo.Cells[0,25] := 'Unused5 (4 longwords)';
  strgdFHBinfo.Cells[0,26] := 'hash chain';
  strgdFHBinfo.Cells[0,27] := 'Parent';
  strgdFHBinfo.Cells[0,28] := 'Extension';
  strgdFHBinfo.Cells[0,29] := 'Secondary type';

  strgdFHBinfo.Cells[1,1]  := formatLong('H', fhb.primtype);
  strgdFHBinfo.Cells[1,2]  := formatLong('H', fhb.header_key)+' ('+formatLong('I', fhb.header_key)+')';
  strgdFHBinfo.Cells[1,3]  := formatLong('H', fhb.high_seq)+' ('+formatLong('I', fhb.high_seq)+')';
  strgdFHBinfo.Cells[1,4]  := formatLong('H', fhb.data_size)+' ('+formatLong('I', fhb.data_size)+')';
  strgdFHBinfo.Cells[1,5]  := formatLong('H', fhb.first_data);
  strgdFHBinfo.Cells[1,6]  := 'Act: '+theADF.block.getBlockChecksum(blockNum)+ '  Calc''d: '+theADF.block.calcBlockChecksum(blockNum);
  strgdFHBinfo.Cells[1,7]  := ''; // Data blocks in listbox
  strgdFHBinfo.Cells[1,8]  := formatLong('H', fhb.unused1);
  strgdFHBinfo.Cells[1,9]  := formatWord('H', fhb.user_id);
  strgdFHBinfo.Cells[1,10] := formatWord('H', fhb.group_id);
  strgdFHBinfo.Cells[1,11] := formatLong('H', fhb.protect_flags)+' ('+theADF.block.getFHBpermissions(blockNum)+')';
  strgdFHBinfo.Cells[1,12] := formatLong('H', fhb.byte_size)+' ('+formatLong('I', fhb.byte_size)+')';
  strgdFHBinfo.Cells[1,13] := formatByte('H', fhb.comm_len)+' ('+formatByte('I', fhb.comm_len)+')';
  strgdFHBinfo.Cells[1,14] := fhb.comment;
  strgdFHBinfo.Cells[1,15] := fhb.unused2;
  strgdFHBinfo.Cells[1,16] := formatLong('I', fhb.days)+getLastChangeDate(fhb.days);
  strgdFHBinfo.Cells[1,17] := formatLong('H', fhb.mins)+' ('+formatLong('I', fhb.mins)+')';
  strgdFHBinfo.Cells[1,18] := formatLong('H', fhb.ticks)+' ('+formatLong('I', fhb.ticks)+')';
  strgdFHBinfo.Cells[1,19] := formatByte('H', fhb.name_len)+' ('+formatByte('I', fhb.name_len)+')';
  strgdFHBinfo.Cells[1,20] := fhb.filename;
  strgdFHBinfo.Cells[1,21] := fhb.unused3;
  strgdFHBinfo.Cells[1,22] := formatLong('H', fhb.unused4);
  strgdFHBinfo.Cells[1,23] := formatLong('H', fhb.real_entry)+' ('+formatLong('I', fhb.real_entry)+')';
  strgdFHBinfo.Cells[1,24] := formatLong('H', fhb.next_link)+' ('+formatLong('I', fhb.next_link)+')';

  strgdFHBinfo.Cells[1,25] := formatLong('H', fhb.unused5[0]);
                            //formatLong('H', fhb.unused5[1]) + ' ' +
                            //formatLong('H', fhb.unused5[2]) + ' ' +
                            //formatLong('H', fhb.unused5[3]);

  strgdFHBinfo.Cells[1,26] := formatLong('H', fhb.hash_chain)+' ('+formatLong('I', fhb.hash_chain)+')';
  strgdFHBinfo.Cells[1,27] := formatLong('H', fhb.parent)+' ('+formatLong('I', fhb.parent)+')';
  strgdFHBinfo.Cells[1,28] := formatLong('H', fhb.extension)+' ('+formatLong('I', fhb.extension)+')';
  strgdFHBinfo.Cells[1,29] := formatLong('H', fhb.sec_type);


  // Populate the datablocks listbox (from the file header block)
  for i := 71 downto 0 do
  begin
    if ((fhb.data_blocks[i]) <> 0) then
      cbDataBlocks.Items.Add(formatLong('I', fhb.data_blocks[i]));
  end;

  // Now check if we need to add to the datablocks listbox by checking if
  // there are any file extension blocks.
  extnBlockPtr := fhb.extension;
  FlipLong(extnBlockPtr);

  while (extnBlockPtr <> 0) do
  begin
    feb := theADF.getFileExtnBlockPtr(extnBlockPtr);

    for i := 71 downto 0 do
    begin
      if ((feb.data_blocks[i]) <> 0) then
        cbDataBlocks.Items.Add(formatLong('I', feb.data_blocks[i]));
    end;
    extnBlockPtr := feb.extension;
    Fliplong(extnBlockPtr);
  end;

end;

procedure TfrmInfomation.cbDataBlocksChange(Sender: TObject);
var
  cb  :TCombobox;
  blockNum   :Integer;
begin
  cb := (Sender As Tcombobox);

  blockNum := StrToInt(cb.Items.Strings[cb.itemindex]);
  dispDBinfo(blockNum);

  // Make sure the portion of the data we are viewing starts on the top row.
  filedataHEXview.TopRow := (cb.ItemIndex * 488) div 16;
  filedataASCIIview.TopRow := fileDataHEXview.TopRow;

  fileDataHexView.Refresh;
  fileDataASCIIView.Refresh;
  
end;

procedure TfrmInfomation.dispDBinfo(blockNum :Integer);
var
  ofsDB       :^T_OFS_datablock;
  //ffsDB       :^T_FFS_datablock;

begin

  if (copy(theADF.getDiskType(), 0, 3) = 'OFS') then
  begin
    ofsDB := theadf.getOFSDataBlockPtr(blockNum);

    strgdDBinfo.Cells[0,0]  := 'OFS Data Block';
    strgdDBinfo.Cells[0,1]  := 'Primary Type';
    strgdDBinfo.Cells[0,2]  := 'Header Key';
    strgdDBinfo.Cells[0,3]  := 'Sequence Num';
    strgdDBinfo.Cells[0,4]  := 'Data Size';
    strgdDBinfo.Cells[0,5]  := 'Next data block ptr';
    strgdDBinfo.Cells[0,6]  := 'Block checksum';

    strgdDBinfo.Cells[1,1]  := formatLong('H', ofsDB.primtype);
    strgdDBinfo.Cells[1,2]  := formatLong('H', ofsDB.header_key)+' ('+formatLong('I', ofsDB.header_key)+')';
    strgdDBinfo.Cells[1,3]  := formatLong('H', ofsDB.seq_num);
    strgdDBinfo.Cells[1,4]  := formatLong('H', ofsDB.data_size)+' ('+formatLong('I', ofsDB.data_size)+')';
    strgdDBinfo.Cells[1,5]  := formatLong('H', ofsDB.next_data)+' ('+formatLong('I', ofsDB.next_data)+')';
    strgdDBinfo.Cells[1,6]  := theADF.block.getBlockChecksum(blockNum) +' ('+theADF.block.calcBlockChecksum(blockNum)+')';
  end
  else
  begin
    strgdDBinfo.Cells[0,0]  := 'FFS Data Block';
    strgdDBinfo.Cells[0,1]  := 'Primary Type';
    strgdDBinfo.Cells[0,2]  := 'Header Key';
    strgdDBinfo.Cells[0,3]  := 'Sequence Num';
    strgdDBinfo.Cells[0,4]  := 'Data Size';
    strgdDBinfo.Cells[0,5]  := 'Next data block ptr';
    strgdDBinfo.Cells[0,6]  := 'Block checksum';

    strgdDBinfo.Cells[1,1]  := 'Not applicable';
    strgdDBinfo.Cells[1,2]  := 'Not applicable';
    strgdDBinfo.Cells[1,3]  := 'Not applicable';
    strgdDBinfo.Cells[1,4]  := 'Not applicable';
    strgdDBinfo.Cells[1,5]  := 'Not applicable';
    strgdDBinfo.Cells[1,6]  := 'Not applicable';
  end;

end;



procedure TfrmInfomation.filedataHexViewDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  pos     :Integer;
  text    :String;
  dg      :TDrawGrid;

begin
  text := theADF.getByteAsHex((0 + (ARow * 16)) + ACol, 'fileMap');
  //DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);

  dg := Sender As TDrawGrid;

  with dg do
  begin
    Canvas.Font.Assign(dg.Font);

    Canvas.Brush.Color := clWindow;

    pos := (Arow * 16) + Acol;

    if (pos >= (cbDataBlocks.ItemIndex * 488))
    and (pos < (cbDataBlocks.ItemIndex * 488) + 488)  then
      Canvas.Brush.Color := clYellow;


    Canvas.Font.Color  := clBlack;

    Canvas.FillRect(Rect);

    ExtTextOut(Canvas.Handle, Rect.Left+ 1, Rect.Top + 1, ETO_CLIPPED or
        ETO_OPAQUE, @Rect, @(text[1]), Length(text), nil);
  end;


end;

procedure TfrmInfomation.filedataAsciiViewDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  text    :String;
  dg      :TDrawGrid;

begin
  text := theADF.getByteAsChr((0 + (ARow * 16)) + ACol, 'fileMap');
  //DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);

  dg := Sender As TDrawGrid;

  with dg do
  begin
    Canvas.Font.Assign(dg.Font);

    Canvas.Brush.Color := clWindow;
    Canvas.Font.Color  := clBlack;

    Canvas.FillRect(Rect);

    ExtTextOut(Canvas.Handle, Rect.Left+ 1, Rect.Top + 1, ETO_CLIPPED or
        ETO_OPAQUE, @Rect, @(text[1]), Length(text), nil);
  end;


end;

procedure TfrmInfomation.filedataAsciiViewTopLeftChanged(Sender: TObject);
begin
  filedataHEXview.TopRow := filedataASCIIview.TopRow;
end;

procedure TfrmInfomation.filedataHexViewTopLeftChanged(Sender: TObject);
begin
  filedataASCIIview.TopRow := filedataHEXview.TopRow;
end;


end.
