unit myFrame;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, ShellAPI, ExtCtrls, ADF_File_Routines, ComCtrls, Menus,
  FileCtrl, Buttons, ImgList;

const
    // We are displaying our adf file in a grid 16 columns wide.
    // An adf format = 16 columns * 56320 rows of data so set up some values
    // for start of rootblock etc..
    BOOTBLOCK_START_ROW = 0;
    BOOTBLOCK_END_ROW   = 63;   // Bootblock = 16 bytes * 64 rows = 1024 bytes
    ROOTBLOCK_START_ROW = 28160;  // (880*512)/16
    ROOTBLOCK_END_ROW   = 28191;  // (880*512)/16 + 31 rows


type
  TADF = class(TFrame)
    pnlHexAsciView: TPageControl;
    TabSheet1: TTabSheet;
    edtFname: TEdit;
    HexView: TDrawGrid;
    AsciiView: TDrawGrid;
    TabSheet2: TTabSheet;
    btnBootBlock: TButton;
    btnRootBlock: TButton;
    lblXYpos: TLabel;
    pnlBootBlockInfo: TPanel;
    strgdBootBlockInfo: TStringGrid;
    lblDosDiskInfo: TLabel;
    pnlDirectoryList: TPanel;
    strgdDirectoryList: TStringGrid;
    lblDirectoryList: TLabel;
    btnGoToBlock: TButton;
    edtGoToBlock: TEdit;
    popMenuSaveFile: TPopupMenu;
    SaveThisFile: TMenuItem;
    SaveAllFiles: TMenuItem;
    pnlDirList: TPanel;
    dlbSaveDir: TDirectoryListBox;
    Label1: TLabel;
    btnOk: TBitBtn;
    btnCancel: TBitBtn;
    TabSheet4: TTabSheet;
    datablocksMemo: TMemo;
    edtFileToInsert: TEdit;
    PopMenuHeaderDetails: TPopupMenu;
    FileHeaderDetails1: TMenuItem;
    Label2: TLabel;
    btnInsertFile: TButton;
    Label3: TLabel;
    Label4: TLabel;
    ImageList1: TImageList;
    SpeedButton1: TSpeedButton;
    tv: TTreeView;
    btnSaveADF: TButton;

    procedure FrameResize(Sender: TObject); // HAS to go here.
    procedure HexViewTopLeftChanged(Sender: TObject);
    procedure AsciiViewTopLeftChanged(Sender: TObject);
    procedure AsciiViewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure DrawCellText(Sender:  TDrawGrid; const text: String;
      ACol, ARow: Longint; ARect :TRect; State: TGridDrawState);
    procedure HexViewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure AsciiViewSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure HexViewSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure btnBootBlockClick(Sender: TObject);
    procedure btnRootBlockClick(Sender: TObject);
    procedure btnGoToBlockClick(Sender: TObject);
    procedure strgdDirectoryListSelectCell(Sender: TObject; ACol,
      ARow: Integer; var CanSelect: Boolean);
    procedure fileHexViewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure fileAsciiViewDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure SaveThisFileClick(Sender: TObject);
    procedure strgdDirectoryListMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure SaveAllFilesClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure FileHeaderDetails1Click(Sender: TObject);

    procedure writetomemo(s :String);
    procedure buildDirTree(pNode :TTreeNode; level  :Integer);
    procedure btnInsertFileClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure btnSaveADFClick(Sender: TObject);

  private

    // Populated in displayRawData function
    m_offset      :LongInt;
    m_mode        :String;
    m_compmode    :String;
    m_tmpFrame    :TADF;

    r             :Integer;
    c             :Integer;
    saveAllFile   :Boolean;

    rootNode      :TTreeNode;

    procedure updXYpos(col, row :Integer);
    procedure dispDosDiskInfo;


  public
    ADF        :TADFlib;

    m_col_clicked   :Integer;
    m_row_clicked   :Integer;

    constructor Create(AOwner :TComponent); override;
    destructor Destroy; override;

    procedure Load(var Msg:  Tmsg);

    procedure displayRawData(mode: String; offset :Longint; Sender: TObject; compmode :String; otherFrame  :TADF);
    function get_diskMap_HEXdata(col, row :Integer) :String;
    function get_diskMap_ASCIIdata(col, row :Integer) :String;
  end; // TADF


implementation

{$R *.dfm}

uses InfoPage, DragDrop;


{===============================================================================
 Constructor
===============================================================================}
constructor TADF.Create(AOwner   :TComponent);
begin
  inherited Create(AOwner);
  m_mode   := '';
  m_offset := 0;
end;

{===============================================================================
 Destructor
===============================================================================}
destructor TADF.Destroy;
begin
  inherited Destroy;
  FreeAndNil(ADF);
end;

{===============================================================================
 Init variables (this procedure called straight after create constructor.
===============================================================================}
procedure TADF.FrameResize(Sender: TObject);
begin
  strgdDirectoryList.Hint     := '';
  strgdDirectoryList.ShowHint := true;
  datablocksMemo.Hide;

  ADF  := TADFlib.create;
  dispDosDiskInfo;  // Display initial default values for the DOS Disk Info page
end;

{===============================================================================
 Called when user loads a given file via drag n drop
===============================================================================}
procedure TADF.Load(var Msg: Tmsg);
const
  BufferLength      :Word = 255;
var
  DroppedFilename   :string;
  FileIndex         :longword;
  QtyDroppedFiles   :word;
  pDroppedFilename  :array [0..255] of Char;
  DroppedFileLength :word;

begin
  FileIndex := $FFFFFFFF;
  QtyDroppedFiles := DragQueryFile(Msg.WParam, FileIndex, pDroppedFilename, BufferLength);
  for FileIndex := 0 to (QtyDroppedFiles - 1) do
  begin
    DroppedFileLength := DragQueryFile(Msg.WParam, FileIndex, pDroppedFilename, BufferLength);
    DroppedFilename   := StrPas(pDroppedFilename);

    // User has dragged a file into the ADF file load box.
    if (Msg.hwnd = Tabsheet1.Handle) then
    begin
      edtFName.Text := DroppedFileName;
      ADF.loadADF(DroppedFilename);
      dispDosDiskInfo;
    end;

    // User has dragged a file into the 'Insert file info ADF box'
    if (Msg.hwnd = edtFileToInsert.Handle) then
    begin
      edtFileToInsert.Text := DroppedFileName;
    end;
  end;
end;

{===============================================================================
 Display the sector data in either Hex or Ascii
===============================================================================}
procedure TADF.displayRawData(mode: String; offset  :Longint; Sender: TObject; compmode :String; otherFrame  :TADF);
begin
  m_offset   := offset;
  m_mode     := mode;          // HEX or ASCII
  m_compmode := compmode;      // NORMAL or COMPARE
  m_tmpFrame := otherFrame;

  (Sender As TDrawGrid).Refresh;
end;

{===============================================================================
 If click on the scrollbar in one of the TDrawGrids then keep the other
 TDrawGrid in sync
===============================================================================}
procedure TADF.HexViewTopLeftChanged(Sender: TObject);
begin
  AsciiView.TopRow := HexView.TopRow;
end;

procedure TADF.AsciiViewTopLeftChanged(Sender: TObject);
begin
  HexView.TopRow := AsciiView.TopRow;
end;

{===============================================================================
 This procedure called if a cell in the AsciiView TDrawGrid component
 needs redrawing
===============================================================================}
procedure TADF.AsciiViewDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  text    :String;
begin
  m_mode := 'ASCII';
  text := get_diskMap_ASCIIdata(ACol, ARow);
  DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);
end;

{===============================================================================
 This procedure called if a cell in the HexView TDrawGrid component
 needs redrawing
===============================================================================}
procedure TADF.HexViewDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  text    :String;
begin
  m_mode := 'HEX';
  text := get_diskMap_HEXdata(ACol, ARow);
  DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);
end;

{===============================================================================
 This procedure actually draws the contents of the current cell
===============================================================================}
procedure TADF.DrawCellText(Sender: TDrawGrid; const text: String; ACol,
  ARow: Integer; ARect: TRect; State: TGridDrawState);

var
  jxw       :String;

begin

  with Sender do
  begin
    Canvas.Font.Assign(Sender.Font);

    if (m_col_clicked = ACol) and (m_row_clicked = ARow) then
    begin
      //=======================================================================
      // We are currently drawing the cell we clicked on
      //=======================================================================
      // Draw a Highlight cursor on current TDrawGrid canvas
      Canvas.Brush.Color := clGreen;
      Canvas.Font.Color  := clWhite;

      if (Sender = HexView) then
      begin
        //Draw Highlight cursor on other TDraw Canvas
        AsciiView.Selection := Selection;
        AsciiView.Canvas.Brush.Color := clRed;
        AsciiView.Canvas.Font.Color  := clWhite;
      end;
      if (Sender = AsciiView) then
      begin
        //Draw Highlight cursor on other TDraw Canvas
        HexView.Selection := Selection;
        HexView.Canvas.Brush.Color := clBlue;
        HexView.Canvas.Font.Color  := clWhite;
      end;

    end
    else
    begin
      //=======================================================================
      // We are drawing all the other cells in the grid.
      //=======================================================================
      if (Sender = hexView) or (Sender = AsciiView) then
      begin
        if m_mode = 'HEX' then
        begin
          if (m_compmode = 'COMPARE') then
          begin
             // We definitely know we are in RightFrame
             // So compare byte against other ADF
             jxw := m_tmpFrame.get_diskMap_HEXdata(ACol, Arow);
             if (text <> jxw) then
             begin
                  Canvas.Brush.Color := clRed;
                  Canvas.Font.Color  := clWhite;
             end
             else
             begin
                  Canvas.Brush.Color := clWindow;
             end;
          end
          else
          begin
            // Left side of screen. Compare Mode = 'NORMAL'
            Canvas.Brush.Color := clWindow;

            // highlight important areas of disc

            // BOOTBLOCK
            if (ARow <= BOOTBLOCK_END_ROW) then
              Canvas.Brush.Color := $0000ee00;

            // ROOTBLOCK
            if (ARow >= ROOTBLOCK_START_ROW) and (ARow <= ROOTBLOCK_END_ROW) then
              Canvas.Brush.Color := $00eeee00;
          end;
          Canvas.FillRect(ARect);
        end;

        if m_mode = 'ASCII' then
        begin
          if (m_compmode = 'COMPARE') then
          begin
             // We definitely know we are in RightFrame
             jxw := m_tmpFrame.get_diskMap_ASCIIdata(ACol, Arow);
             if (text <> jxw) then
             begin
                  Canvas.Brush.Color := clRed;
                  Canvas.Font.Color  := clWhite;
             end
             else
             begin
                  Canvas.Brush.Color := clWindow;
             end;
          end
          else
          begin
            // Compare Mode = 'NORMAL'
            Canvas.Brush.Color := clWindow;
          end;
          Canvas.FillRect(ARect);
        end;
      end;
    end;

    ExtTextOut(Canvas.Handle, ARect.Left+ 1, ARect.Top + 1, ETO_CLIPPED or
      ETO_OPAQUE, @ARect, @(text[1]), Length(text), nil);
  end;
end;

{===============================================================================
 Sets the modular variable so know what cell we last clicked.
===============================================================================}
procedure TADF.AsciiViewSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  m_col_clicked := ACol;
  m_row_clicked := ARow;
  updXYpos(m_col_clicked, m_row_clicked);
end;

{===============================================================================
 Sets the modular variable so know what cell we last clicked.
===============================================================================}
procedure TADF.HexViewSelectCell(Sender: TObject; ACol, ARow: Integer;
  var CanSelect: Boolean);
begin
  m_col_clicked := ACol;
  m_row_clicked := ARow;
  updXYpos(m_col_clicked, m_row_clicked);
end;

{===============================================================================
 Return the value in the diskMap corresponding to its X,Y position in the
 HEX Grid
===============================================================================}
function TADF.get_diskMap_HEXdata(col, row: Integer): String;
begin
  Result := ADF.getByteAsHex((m_offset + (row * 16)) + col, 'diskMap');

end;

{===============================================================================
 Return the value in the diskMap corresponding to its X,Y position in the
 ASCII Grid
===============================================================================}
function TADF.get_diskMap_ASCIIdata(col, row: Integer): String;
begin
  Result := ADF.getByteAsChr((m_offset + (row * 16)) + col, 'diskMap');
end;

{===============================================================================
 Go to the start of the BootBlock data
===============================================================================}
procedure TADF.btnBootBlockClick(Sender: TObject);
begin
  AsciiView.TopRow := BOOTBLOCK_START_ROW;
  HexView.TopRow   := BOOTBLOCK_START_ROW;
end;

{===============================================================================
 Go to the start of the RootBlock data
===============================================================================}
procedure TADF.btnRootBlockClick(Sender: TObject);
begin
  AsciiView.TopRow := ROOTBLOCK_START_ROW;
  HexView.TopRow   := ROOTBLOCK_START_ROW;
end;

{===============================================================================
 Go to the start of block user asks for
===============================================================================}
procedure TADF.btnGoToBlockClick(Sender: TObject);
begin
  AsciiView.TopRow := (StrToInt(edtGoToBlock.Text) * 512) DIV 16;
  HexView.TopRow   := AsciiView.TopRow;
end;

{===============================================================================
 Debug - Display X,Y position of last cell in the grid that was clicked in
===============================================================================}
procedure TADF.updXYpos(col, row :Integer);
begin
  lblXYpos.caption := 'Col='+IntToStr(col)+' Row='+IntToStr(row);
end;

{===============================================================================
 Display generic information about the disk image
===============================================================================}
procedure TADF.dispDosDiskInfo;
var
  i, r     :Integer;
  dl       :TDirectoryEntry;

begin
  strgdBootBlockInfo.Cells[0,0] := 'File Name';
  strgdBootBlockInfo.Cells[0,1] := 'File System';
  strgdBootBlockInfo.Cells[0,2] := 'Disk Type';
  strgdBootBlockInfo.Cells[0,3] := 'Bootblock Checksum';
  strgdBootBlockInfo.Cells[0,4] := 'Bootblock Calculated Checksum';
  strgdBootBlockInfo.Cells[0,5] := 'Disk Label';

  strgdBootBlockInfo.Cells[1,0] := ADF.getADFWindowsFileName;
  strgdBootBlockInfo.Cells[1,1] := ADF.getFileSystem;
  strgdBootBlockInfo.Cells[1,2] := ADF.getDiskType;

  // Will crash! if forget to check if fileloaded.
  if (ADF.FileLoaded) then
  begin
    strgdBootBlockInfo.Cells[1,3] := ADF.block.getBlockChecksum(0);
    strgdBootBlockInfo.Cells[1,4] := ADF.block.calcBlockChecksum(0);
  end
  else
  begin
    strgdBootBlockInfo.Cells[1,3] := 'No File Loaded';
    strgdBootBlockInfo.Cells[1,4] := 'No File Loaded';
  end;


  strgdBootBlockInfo.Cells[1,5] := ADF.getDiskLabel;

  // Directory/File list (DOS Disk Info Tab)

  // Clear down the list first
  for r := 0 to strgdDirectoryList.RowCount do
  begin
    strgdDirectoryList.Cells[0, r] := '';
    strgdDirectoryList.Cells[1, r] := '';
    strgdDirectoryList.Cells[2, r] := '';
    strgdDirectoryList.Cells[3, r] := '';
  end;

  strgdDirectoryList.Cells[0,0] := 'Parent Dir';
  strgdDirectoryList.Cells[1,0] := 'Filename';
  strgdDirectoryList.Cells[2,0] := 'Length (bytes)';
  strgdDirectoryList.Cells[3,0] := 'Header Block';

  if (ADF.dirList <> nil) then
  begin
    if (ADF.dirlist.Count > 0) then
      // Set the new rowcount
      strgdDirectoryList.RowCount := ADF.dirList.Count;

    for i := 0 to ADF.dirList.Count-1 do
    begin
      dl := (ADF.dirList.Objects[i] as TDirectoryEntry);
      strgdDirectoryList.Cells[0,i+1] := IntToStr(dl.level) + ' : ' + dl.parentdir_name;
      strgdDirectoryList.Cells[1,i+1] := dl.name;
      strgdDirectoryList.Cells[2,i+1] := IntToStr(dl.size_bytes);
      strgdDirectoryList.Cells[3,i+1] := IntToStr(dl.blocknum);
    end;
  end;

  // Populate Directory/File TTreeView (Insert File tab)

  if ADF.FileLoaded then
  begin
    tv.Items.Clear;
    rootNode := tv.Items.addFirst(nil, 'ROOT');   // Add Root Node (level 0) as ROOT at first.
    rootNode.Data := pointer(1);                  // Mark it as a directory.
    buildDirTree(rootNode, 1);
                        // Build the directory tree (starting at level 1)
    //rootnode.Text := ADF.getDiskLabel;            // Update Root Node to Disk Name
  end
  else
  begin
    tv.Items.Clear;
    rootNode := tv.Items.addFirst(nil, 'ADF image not loaded.');
  end;
end;


procedure TADF.buildDirTree(pNode :TTreeNode; level: Integer);
var
  n         :Integer;
  nLevel    :Integer;
  aNode     :TTreeNode;
  dl        :TDirectoryEntry;

begin
  // Level 0 = Root Node (Must be blank but can be set at end of this process
  // Level 1 = Top level of directories..   c, l, s, devs etc..

  // Go through all the directories in the list with the current level,
  // and if the directories parent matches pNode.text passed in then
  // add it as a child.

  for n := 0 to ADF.dirList.Count-1 do
  begin
    dl := (ADF.dirList.Objects[n] as TDirectoryEntry);

    if (dl.level = level)
    and (dl.parentdir_name = pNode.Text) then
    begin
      // Add this directory
      aNode := tv.Items.addchild(pNode, dl.name);

      if (dl.DirFileRoot = 'F') then
      begin
        // File graphic
        aNode.ImageIndex    := 2;
        aNode.SelectedIndex := 2;
      end;

      if (dl.DirFileRoot = 'D') then
      begin
        // Directory graphic
        aNode.ImageIndex    := 0;
        aNode.SelectedIndex := 1;
        aNode.Data := pointer(1);     // Mark it as directory
      end;

      nLevel := level + 1;
      buildDirTree(aNode, nLevel);
    end;
  end;
end;




procedure TADF.strgdDirectoryListSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
var
  fd    :TDirectoryEntry;

begin

  if (ARow >= 0) and (ARow <= ADF.dirList.count) then
  begin
    fd := (ADF.dirList.Objects[Arow-1] as TDirectoryEntry);

    ADF.extractFile(fd.blocknum, 'FileHeader');
    StrgdDirectoryList.Hint := 'Click a file to see a Hex/ASCII view of its data in the File Info tab';
  end
  else
    StrgdDirectoryList.Hint := 'Nothing to see here.';
end;


procedure TADF.fileHexViewDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  text    :String;
begin
  m_mode := 'HEX';

  text := ADF.getByteAsHex((m_offset + (ARow * 16)) + ACol, 'fileMap');
  DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);
end;

procedure TADF.fileAsciiViewDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
var
  text    :String;

begin
  m_mode := 'ASCII';
  text := ADF.getByteAsChr((m_offset + (ARow * 16)) + ACol, 'fileMap');
  DrawCellText(Sender As TDrawGrid, text, ACol, ARow, Rect, State);
end;

procedure TADF.strgdDirectoryListMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  n,n2      :Integer;
  rec       :TRect;

  mouse     :TMouse;
  mouseX    :Integer;
  mouseY    :Integer;

begin

  mouseX := mouse.CursorPos.X;
  mouseY := mouse.CursorPos.Y;
  r := 0;
  c := 0;

  if (Button = mbRight) then
  begin
    // User clicked the right mouse button on the directory list screen
    for n := 0 to ADF.dirList.Count do
    begin
      rec := strgdDirectoryList.CellRect(0,n);
      if (Y > rec.Top) and (Y < rec.Bottom) then
      begin
        r := n;

        for n2 := 0 to 3 do
        begin
          rec := strgdDirectoryList.CellRect(n2, r);
          // Now find out which column we clicked in
         if (X > rec.Left) and (X < rec.Right) then
         begin
            c := n2;
            //showMessage('You clicked in cell (row='+IntToStr(r)+') (column='+IntToStr(c)+')');
            break;
         end;
        end;

        //break;
      end;
    end;
    // At this point
    // 'r' and 'c' will equal row/column we clicked in.
    // the value contained in 'r' (row) and 'c' (column) is used in the btnOkClick function below.

    if ((r > 0) and (r <= ADF.dirList.Count)) then
    begin
      if (c = 1) then
        PopMenuSaveFile.Popup(mouseX, mouseY);

      if (c = 3) then
        PopMenuHeaderDetails.Popup(mouseX, mouseY);
    end;
  end;
end;


// 'Save File' option on popup menu
procedure TADF.SaveThisFileClick(Sender: TObject);
begin
    saveAllFile := false;
    pnlDirList.Visible := true;

end;

procedure TADF.SaveAllFilesClick(Sender: TObject);
begin
    saveAllFile := true;
    pnlDirList.Visible := true;
end;


procedure TADF.btnOkClick(Sender: TObject);
var
  n       :Integer;
  dest    :string;
  dl      :TDirectoryEntry;

begin
  dest := dlbSaveDir.Directory;

  if (copy(dest, length(dest), 1) <> '\') then
    dest := dest + '\';

  if (saveAllFile = true) then
  begin
    for n := 0 to ADF.dirList.Count-1 do
    begin
      dl := (ADF.dirList.Objects[n] as TDirectoryEntry);

      if (dl.DirFileRoot = 'F') then
      begin
        ADF.extractFile(dl.blocknum, 'FileHeader');
        ADF.SaveFile(dest+ dl.name, dl.size_bytes);
      end;
    end;
  end
  else
  begin
    dl := (ADF.dirList.Objects[r-1] as TDirectoryEntry);
    ADF.extractFile(dl.blocknum, 'FileHeader');
    ADF.SaveFile(dest+ dl.name, dl.size_bytes);
  end;

  pnlDirList.Visible := false;

end;

procedure TADF.btnCancelClick(Sender: TObject);
begin
  pnlDirList.Visible := false;
end;


procedure TADF.FileHeaderDetails1Click(Sender: TObject);
var
  frm         :TfrmInfomation;
  dl          :TDirectoryEntry;

begin
  dl := (ADF.dirList.Objects[r-1] as TDirectoryEntry);
  frm := TfrmInfomation.Create(self);
  frm.initInfoPage(ADF, dl.blockNum);
  frm.ShowModal;

end;

procedure TADF.writetomemo(s: String);
begin
  datablocksMemo.Lines.Add(s);
end;



procedure TADF.btnInsertFileClick(Sender: TObject);
var
  pNode             :TTreeNode;
  cNode             :TTreeNode;
  dupeExist         :Boolean;
  isDir             :Integer;
  fileToInsert      :String;
  dirToInsertInto   :String;

begin

  if (ADF.FileLoaded = false) then
  begin
    showMessage('No ADF image loaded');
    Exit;
  end;

  if (edtFileToInsert.Text = '') then
  begin
    showMessage('No file to insert specified');
    Exit;
  end;

  if (tv.Selected = nil) then
  begin
    showMessage('You must select a directory node to insert file into');
    Exit;
  end;

  pNode := tv.selected;
  isDir := integer(pNode.Data);

  if (isdir <> 1) then
  begin
    showMessage('You must select a directory node to insert file into');
    Exit;
  end;

  // If got to here then check that the file we are inserting doesnt already
  // exist in the ADF image.
  fileToInsert    := extractFileName(edtFileToInsert.Text);

  cNode     := pNode.getFirstChild;
  dupeExist := false;

  if (cNode <> nil) then
  begin
    while cNode <> nil do
    begin
      isDir := integer(cNode.data);
      // Skip check if cNode is a directory
      if (isDir <> 1) then
      begin
        if (cNode.Text = fileToInsert) then
        begin
          dupeExist := true;
          break;
        end;
      end;
      cNode := cNode.getNextSibling;
    end;
  end;

  if (dupeExist = true) then
  begin
    showMessage('A duplicate file already exists in the selected directory');
    Exit;
  end;

  // If got to here then all params ok..
  dirToInsertInto := tv.selected.Text;

  //showMessage('Inserting: '+fileToInsert+' into directory: '+dirToInsertInto);

  ADF.insertFileIntoADFimage(writetomemo, dirToInsertInto, edtFileToInsert.Text);
  dispDosDiskInfo;

end;

procedure TADF.SpeedButton1Click(Sender: TObject);
begin
  if not (datablocksMemo.Showing) then
  begin
    tv.Hide;
    dataBlocksMemo.Show;
  end
  else
  begin
    tv.Show;
    datablocksMemo.Hide;
  end;
end;

procedure TADF.btnSaveADFClick(Sender: TObject);
begin
  ADF.SaveADF('d:\newadf.adf', 901120);
end;

end.
