unit ADF_diff_funcs;

interface

uses
  SysUtils, Classes, Dialogs;


type

  // An 'entry' is identified by its name 000010, 000011 etc..
  // and contains a list of start positions

  // The name is actually the number of bytes to copy from the start position.

  // An EntryList holds a list of Entries

  TEntry = class(TObject)
    private
      offsetList     : TList;

    protected
    public

      constructor Create;
      destructor Destroy; override;
      procedure setOffsetItemValue(pos :Longint);
      function  getOffsetListCount : integer;
      function  getOffsetItemValue(idx : integer) : LongInt;
  end;


  TEntryList = class(TStringList)
    private
      entry   : TEntry;
    protected
    public
      constructor Create;
      destructor  Destroy; override;

      // nbytes is converted into an entryName
      procedure InsertEntry(nbytes : LongInt; spos : Longint);

      function getOffSetListCount(idx : integer) : integer;
      function  getOffsetItemValue(idx : integer; idx2 : integer) : LongInt;


  end;




implementation

{ TEntry }

constructor TEntry.Create;
begin
  offsetList := TList.Create;
end;


destructor TEntry.Destroy;
begin
  while offsetList.count > 0 do
    offsetList.Delete(0);
  FreeAndNil(offsetList);
  inherited;
end;

procedure TEntry.setOffsetItemValue(pos :Longint);
begin
  offsetList.Add(Pointer(pos));
end;

function TEntry.getOffsetListCount: integer;
begin
  Result := offsetList.Count;
end;

function TEntry.getOffsetItemValue(idx: integer): LongInt;
begin
  Result := LongInt(self.offsetList.Items[idx]);
end;



{ TEntryList }

constructor TEntryList.Create;
begin
  inherited;
end;

destructor TEntryList.Destroy;
begin
  inherited;
end;


function TEntryList.getOffsetListCount(idx: integer): integer;
var
  entry   :TEntry;
begin
  // This list is a list of TEntry items.
  entry := TEntry(Objects[idx]);      // Get the TEntry item we want.
  Result := entry.getOffsetListCount;
end;

function TEntryList.getOffsetItemValue(idx: integer; idx2 : integer): LongInt;
var
  entry   :TEntry;
begin
  entry := TEntry(Objects[idx]);
  Result := entry.getOffsetItemValue(idx2);
end;

procedure TEntryList.InsertEntry(nbytes, spos: LongInt);
var
  s   : string;
  i   : integer;
begin
  // If the entry 'nbytes' already exists then just add the spos to it
  // else creare the entry nbytes and add spos to it.

  s := Format('%.6d', [nbytes]);

  i := IndexOf(s);
  if (i = -1) then
  begin
    // Create the Entry record.
    entry := TEntry.Create;
    entry.setOffsetItemValue(spos);
    // Add the record to our list
    AddObject(s, entry);
  end
  else
  begin
    entry := TEntry(GetObject(i));
    entry.setOffsetItemValue(spos);
  end;


end;



end.
 