unit general_funcs;

interface

uses
  SysUtils, Controls;

    procedure FlipLong(var l :longword);
    function formatLong(t :String; lw :longword) : string;
    function formatWord(t :String; w: Word) : string;
    function formatByte(t :String; b: Byte) : string;
    function getLastChangeDate(days :Longword) :String;

implementation

{==========================================================================
 FUNCTION:    Fliplong
 DESCRIPTION: Flips a longword value from Intel format to Motorola fomat
 ACCEPTS:     longword value
 RETURNS:     flipped longword value
==========================================================================}
procedure FlipLong(var l: longword);
var
  b1:array[0..3] of byte;
  b2:array[0..3] of byte;

begin
  move(l,b1,4);
  b2[0]:=b1[3];
  b2[1]:=b1[2];
  b2[2]:=b1[1];
  b2[3]:=b1[0];
  move(b2,l,4);
end;

{==========================================================================
 FUNCTION:    formatLong
 DESCRIPTION: Formats a longword into a string
 ACCEPTS:     return type (H or I), longword value
 RETURNS:     longword value as a string in HEX or Integer format
==========================================================================}
function formatLong(t: String; lw: longword): string;
var
  tmp   :longword;
begin
  tmp := lw;
  Fliplong(tmp);

  if (t = 'H') then
    Result := '$' + Format('%0.8x', [tmp]);

  if (t = 'I') then
    Result := IntToStr(tmp);
end;

{==========================================================================
 FUNCTION:    formatWord
 DESCRIPTION: Formats a Word into a string
 ACCEPTS:     return type (H or I), word value
 RETURNS:     Word value as a string in HEX or Integer format
==========================================================================}
function formatWord(t: String; w: Word): string;
var
  tmp   :Word;
begin
  tmp := w;

  if (t = 'H') then
    Result := '$' + Format('%0.4x', [tmp]);

  if (t = 'I') then
    Result := IntToStr(tmp);
end;

{==========================================================================
 FUNCTION:    formatByte
 DESCRIPTION: Formats a Byte into a string
 ACCEPTS:     return type (H or I), Byte value
 RETURNS:     Byte value as a string in HEX or Integer format
==========================================================================}
function formatByte(t: String; b: Byte): string;
var
  tmp   :Byte;
begin
  tmp := b;

  if (t = 'H') then
    Result := '$' + Format('%0.2x', [tmp]);

  if (t = 'I') then
    Result := IntToStr(tmp);
end;


{==========================================================================
 FUNCTION:    getLastChangeDate
 DESCRIPTION: Returns the date the disk/file was last changed
 ACCEPTS:     Number of days since 01/Jan/1978
 RETURNS:     01/01/1978 + number of days
==========================================================================}
function getLastChangeDate(days :Longword) :String;
var
  theDate   :TDateTime;
begin
  Fliplong(days);
  theDate := TDatetime(StrToDate('01/01/1978'));
  theDate := theDate + days;
  Result := DateToStr(theDate);
end;




end.

