unit U_Dialog_DatabaseEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, U_ExChange, U_Dialog_DiskEdit,
  U_Dialog_FileEdit, U_Dialog_GameEdit, U_Dialog_RangeEdit,
  U_Dialog_VersionEdit, ImgList;

var Open_DatabaseEdit:record
                        filename:string;
                        createnew:boolean;
                        closedp:pointer;
                      end;

type
  TF_DatabaseEdit = class(TForm)
    DE_Standard: TGroupBox;
    ST_Name: TStaticText;
    ED_Name: TEdit;
    ED_Creator: TEdit;
    ED_RelDay: TEdit;
    ED_RelMonth: TEdit;
    ED_RelYear: TEdit;
    ST_Creator: TStaticText;
    ST_RelDate: TStaticText;
    DE_Records: TGroupBox;
    TV_Records: TTreeView;
    B_NewGame: TButton;
    B_NewVersion: TButton;
    B_NewDisk: TButton;
    B_Edit: TButton;
    B_Delete: TButton;
    B_Okay: TButton;
    B_Save: TButton;
    B_Cancel: TButton;
    ImageList1: TImageList;
    L_Stats: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure B_NewGameClick(Sender: TObject);
    procedure B_NewVersionClick(Sender: TObject);
    procedure B_NewDiskClick(Sender: TObject);
    procedure B_EditClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure B_OkayClick(Sender: TObject);
    procedure B_SaveClick(Sender: TObject);
    procedure B_CancelClick(Sender: TObject);
    procedure B_DeleteClick(Sender: TObject);
    procedure TV_RecordsKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    db:DB_Base;
    lastdata:pointer;
    usefilename:string;
    SaveOnExit:boolean;
  public
    saveenable:boolean;
    procedure RecordsToScreen;
    procedure DBToScreen;
    procedure ScreenToDB;
    { Public declarations }
  end;

var
  F_DatabaseEdit: TF_DatabaseEdit;

implementation

{$R *.dfm}

procedure TF_DataBaseEdit.RecordsToScreen;
var GN,TN:TTreeNode;
    TDS:DB_Soft;
    TDD:DB_Disk;
    TDV:DB_Version;
    c:longword;
    s:string;
    numg,numv,numd,numm:longword;
begin
  TV_Records.Items.Clear;
  numg:=0;
  numv:=0;
  numd:=0;
  numm:=0;

  TDS:=DB^.Softs;

  while TDS<>NIL do begin
    inc(numg);
    s:=TDS.GameName^+' ('+TDS^.Publisher^+' / '+IntToStr(TDS^.Releaseyear)+')';
    if GetPSString(TDS^.comment)<>'' then s:=s+' ['+GetPSString(TDS^.comment)+']';
    GN:=TV_Records.Items.Add(NIL,s);
    GN.ImageIndex:=0;
    GN.SelectedIndex:=0;
    GN.Data:=TDS;
    TDV:=TDS^.Versions;
    c:=1;
    while TDV<>NIL do begin
      inc(numv);
      s:='V#'+IntToStr(c)+': ';
      s:=s+GetVersionNameEx(TDS,TDV);
      if TDV^.comment<>NIL then s:=s+' ('+GetPSString(TDV^.comment)+')';
      TN:=TV_Records.Items.AddChild(GN,s);
      TN.Data:=TDV;
      TN.ImageIndex:=1;
      TN.SelectedIndex:=1;
      TDV:=TDV.Next;
      inc(c);
    end;
    TDD:=TDS^.Disks;
    c:=1;
    while TDD<>NIL do begin
      inc(numd);
      s:='D#'+IntToStr(c)+': ';
      s:=s+GetDiskName{Ex}(TDS,TDD);    { test0001 }
      if TDD^.comment<>NIL then s:=s+' ('+GetPSString(TDD^.comment)+')';
      TN:=TV_Records.Items.AddChild(GN,s);
      TN.Data:=TDD;
      TN.ImageIndex:=2;
      TN.SelectedIndex:=2;
      TDD:=TDD.Next;
      inc(c);
    end;
    TDS:=TDS^.next;
  end;
  TV_Records.AlphaSort(FALSE);
  TV_Records.FullCollapse;
  numm:=numg*(sizeof(TDS^)+96)+numv*(sizeof(TDV^)+128)+numd*(sizeof(TDD^)+512);
  L_Stats.Caption:=IntToStr(numg)+' games, '+
                   IntToStr(numv)+' versions, '+
                   IntToStr(numd)+' disks, ~'+
                   IntToStr(numm div 1024)+'kb mem used';
end;

procedure TF_DataBaseEdit.DBToScreen;
begin
  if DB.creator<>NIL then ED_Creator.Text:=DB.creator^;
  if DB.name<>NIL then ED_Name.Text:=DB.name^;
  ED_RelYear.Text:=IntToStr(DB.releaseyear);
  ED_RelMonth.Text:=IntToStr(DB.releasemonth);
  ED_RelDay.Text:=IntToStr(DB.releaseday);
end;

procedure TF_DataBaseEdit.ScreenToDB;
begin
  SetPSString(DB.creator,ED_Creator.Text);
  SetPSString(DB.name,ED_Name.Text);
  DB^.releaseyear:=StrToInt(ED_RelYear.Text);
  DB^.releasemonth:=StrToInt(ED_RelMonth.Text);
  DB^.releaseday:=StrToInt(ED_RelDay.Text);
end;

procedure TF_DatabaseEdit.FormCreate(Sender: TObject);
begin
  SaveOnExit:=FALSE;
  usefilename:=open_databaseedit.filename;
  If Open_DataBaseEdit.createnew then begin
    new(DB);
    fillchar(DB^,sizeof(DB^),0);
    SetPSString(DB.name,'New Database ['+usefilename+']');
    SetPSString(DB.creator,'New Creator');
  end else begin
    DIDLoad(DB,usefilename);
  end;
  DBToScreen;
  RecordsToScreen;
end;

procedure TF_DatabaseEdit.FormClose(Sender: TObject;
  var Action: TCloseAction);
var DBS,DBSN:DB_Soft;
    DBD,DBDN:DB_Disk;
    DBV,DBVN:DB_Version;
    DBF,DBFN:DB_File;
    DBR,DBRN:DB_Range;
begin
  ScreenToDB;
  if (SaveOnExit) and (saveenable) then DIDSave(DB,usefilename);
  DEL_Base(DB);
{  if DB.creator<>NIL then freemem(DB.creator,length(DB.creator^)+1);
  if DB.name<>NIL then freemem(DB.name,length(DB.name^)+1);
  DBS:=DB.Softs;
  while DBS<>NIL do begin
    DBSN:=DBS.next;
    DBV:=DBS.versions;
    while DBV<>NIL do begin
      DBVN:=DBV.next;
      dispose(DBV);
      DBV:=DBVN;
    end;
    DBD:=DBS.disks;
    while DBD<>NIL do begin
      DBDN:=DBD.next;
      DBF:=DBD.files;
      while DBF<>NIL do begin
        DBFN:=DBF.next;
        dispose(DBF);
        DBF:=DBFN;
      end;
      DBR:=DBD.ranges;
      while DBR<>NIL do begin
        DBRN:=DBR.next;
        dispose(DBR);
        DBR:=DBRN;
      end;
      dispose(DBD);
      DBD:=DBDN;
    end;
    dispose(DBS);
    DBS:=DBSN;
  end;
  dispose(DB);}
  Release;
end;

procedure TF_DatabaseEdit.B_NewGameClick(Sender: TObject);
var DBS:DB_Soft;
    c:longword;
    GE:TF_GameEdit;
begin
  if DB.Softs=NIL then begin
    New(DB.softs);
    DBS:=DB.softs;
  end else begin
    DBS:=DB.softs;
    while DBS^.next<>NIL do DBS:=DBS^.next;
    new(DBS^.next);
    DBS:=DBS^.next;
  end;
  fillchar(DBS^,sizeof(DBS^),$00);
  SetPSString(DBS^.gamename,'[New Game]');
  DBS^.releaseyear:=0;
  DBS^.releasemonth:=0;
  DBS^.releaseday:=0;
  SetPSString(DBS^.publisher,'Publisher');
  open_gameedit_data:=DBS;
  GE:=TF_GameEdit.Create(self);
  GE.Show;
  lastdata:=DBS;
{  RecordsToScreen;
  c:=0;
  TV_Records.AlphaSort(FALSE);
  while TV_Records.Items[c].Data<>DBS do inc(c);
  TV_Records.TopItem:=TV_Records.Items[c];}
end;

procedure TF_DatabaseEdit.B_NewVersionClick(Sender: TObject);
var DBV:DB_Version;
    DBS:DB_Soft;
    c:longword;
    VE:TF_VersionEdit;
    NTN,TN:TTreeNode;
    vn:longword;
begin
  TN:=TV_Records.Selected;
  if TN<>NIL then begin
    if TN.Level=0 then begin
        DBS:=TN.Data;
      if DBS.Versions=NIL then begin
        New(DBS.Versions);
        DBV:=DBS.Versions;
      end else begin
        DBV:=DBS.Versions;
        while DBV^.next<>NIL do DBV:=DBV^.next;
        new(DBV^.next);
        DBV:=DBV^.next;
      end;
      fillchar(DBV^,sizeof(DBV^),$00);
      SetPSString(DBV^.VersionName,'[New Version]');
      DBV^.releaseyear:=0;
      DBV^.releasemonth:=0;
      DBV^.releaseday:=0;
      DBV^.memc:=1024;
      DBV^.lang:=$00000001;
      DBV^.compat.cpu:=$0aaa;
      DBV^.compat.chipset:=$0028;
      DBV^.compat.floppy:=$10aa;
      DBV^.compat.graphics:=$002a;
      DBV^.compat.memchip:=512;
      DBV^.compat.memslow:=512;
      DBV^.compat.kickstart:=$0aaa;
      DBV^.compat.special:=$5008;
      DBV^.compat.winuae:=$11608002;
      DBV^.compat.winfellow:=$150b0f02;
      open_versionedit_data:=DBV;
      VE:=TF_VersionEdit.Create(self);
      VE.Show;
      lastdata:=DBS;
{      RecordsToScreen;
      c:=0;
      TV_Records.AlphaSort(FALSE);
      while TV_Records.Items[c].Data<>DBS do inc(c);
      TV_Records.TopItem:=TV_Records.Items[c];}
    end;
  end;
end;

procedure TF_DatabaseEdit.B_NewDiskClick(Sender: TObject);
var DBD:DB_Disk;
    DBS:DB_Soft;
    c:longword;
    DE:TF_DiskEdit;
    TN:TTreeNode;
begin
  TN:=TV_Records.Selected;
  if TN<>NIL then begin
    if TN.Level=0 then begin
        DBS:=TN.Data;
      if DBS.Disks=NIL then begin
        New(DBS.Disks);
        DBD:=DBS.Disks;
      end else begin
        DBD:=DBS.Disks;
        while DBD^.next<>NIL do DBD:=DBD^.next;
        new(DBD^.next);
        DBD:=DBD^.next;
      end;
      fillchar(DBD^,sizeof(DBD^),$00);
      SetPSString(DBD^.DiskName,'[New Disk]');
      open_diskedit_data:=DBD;
      open_diskedit_parent:=DBS;
      DE:=TF_DiskEdit.Create(self);
      DE.Show;
      lastdata:=DBS;
{      RecordsToScreen;
      c:=0;
      TV_Records.AlphaSort(FALSE);
      while TV_Records.Items[c].Data<>DBS do inc(c);
      TV_Records.TopItem:=TV_Records.Items[c];}
    end;
  end;
end;

procedure TF_DatabaseEdit.B_EditClick(Sender: TObject);
var c:longword;
    DE:TF_DiskEdit;
    VE:TF_VersionEdit;
    GE:TF_GameEdit;
    TN:TTreeNode;
    TN2:TTreeNode;
begin
  TN:=TV_Records.Selected;
  if TN<>NIL then begin
    if TN.Level=1 then begin
      if TN.Text[1]='V' then begin
        open_versionedit_data:=TN.Data;
        VE:=TF_VersionEdit.Create(self);
        VE.Show;
        lastdata:=TN.Data;
      end else begin
        open_diskedit_data:=TN.Data;
        open_diskedit_parent:=TN.Parent.Data;
        DE:=TF_DiskEdit.Create(self);
        DE.Show;
        lastdata:=TN.Data;
      end;
    end else begin
      open_gameedit_data:=TN.Data;
      GE:=TF_GameEdit.Create(self);
      GE.Show;
      lastdata:=TN.Data;
    end;
  end;
{  RecordsToScreen;
  c:=0;
  TV_Records.AlphaSort(FALSE);
  while TV_Records.Items[c].Data<>TN.Data do inc(c);
  TV_Records.TopItem:=TV_Records.Items[c];}
end;

procedure TF_DatabaseEdit.FormActivate(Sender: TObject);
var c:longword;
    n:byte;
    paintagain:boolean;
begin
  paintagain:=FALSE;
  for n:=$00 to $ff do if L_GameEdit[n].ready then begin
    L_GameEdit[n].inst:=NIL;
    L_GameEdit[n].ready:=FALSE;
    paintagain:=TRUE;
  end;
  for n:=$00 to $ff do if L_VersionEdit[n].ready then begin
    L_VersionEdit[n].inst:=NIL;
    L_VersionEdit[n].ready:=FALSE;
    paintagain:=TRUE;
  end;
  for n:=$00 to $ff do if L_DiskEdit[n].ready then begin
    L_DiskEdit[n].inst:=NIL;
    L_DiskEdit[n].ready:=FALSE;
    paintagain:=TRUE;
  end;
  if paintagain then begin
    RecordsToScreen;
    c:=0;
    while (TV_Records.Items[c].Data<>lastdata) and (c<TV_Records.Items.count) do inc(c);
    if c<TV_Records.Items.Count then begin
      TV_Records.TopItem:=TV_Records.Items[c];
      TV_Records.Select([TV_Records.Items[c]]);
      TV_Records.SetFocus;
    end;
  end;
end;

procedure TF_DatabaseEdit.B_OkayClick(Sender: TObject);
begin
  SaveOnExit:=TRUE;
  Close;
end;

procedure TF_DatabaseEdit.B_SaveClick(Sender: TObject);
begin
  if saveenable then DIDSave(DB,usefilename);
end;

procedure TF_DatabaseEdit.B_CancelClick(Sender: TObject);
begin
  Close;
end;

procedure TF_DatabaseEdit.B_DeleteClick(Sender: TObject);
var c:longword;
    DE:TF_DiskEdit;
    VE:TF_VersionEdit;
    GE:TF_GameEdit;
    TN:TTreeNode;
    TN2:TTreeNode;
    TDV,TTDV:DB_Version;
    TDS,TTDS:DB_Soft;
    TDD,TTDD:DB_Disk;
begin
  TN:=TV_Records.Selected;
  if TN<>NIL then begin
    if TN.Level=1 then begin
      if TN.Text[1]='V' then begin
        TDV:=TN.Data;
        TDS:=TN.Parent.Data;
        if TDS^.versions=TDV then TDS^.versions:=TDV^.next else begin
          TTDV:=TDS^.versions;
          while TTDV^.next<>TDV do TTDV:=TTDV^.next;
          TTDV^.next:=TDV^.next;
        end;
        DEL_VersionContent(TDV);
        dispose(TDV);
        TN.Delete;
      end else begin
        TDD:=TN.Data;
        TDS:=TN.Parent.Data;
        if TDS^.disks=TDD then TDS^.disks:=TDD^.next else begin
          TTDD:=TDS^.disks;
          while TTDD^.next<>TDD do TTDD:=TTDD^.next;
          TTDD^.next:=TDD^.next;
        end;
        DEL_DiskContent(TDD);
        dispose(TDD);
        TN.Delete;
      end;
    end else begin
      TDS:=TN.Data;
      if DB^.softs=TDS then DB^.softs:=TDS^.next else begin
        TTDS:=DB^.softs;
        while TTDS^.next<>TDS do TTDS:=TTDS^.next;
        TTDS^.next:=TDS^.next;
      end;
      DEL_SoftContent(TDS);
      dispose(TDS);
      TN.Delete;
    end;
  end;
end;

procedure TF_DatabaseEdit.TV_RecordsKeyPress(Sender: TObject;
  var Key: Char);
begin
  Beep;
end;

end.
