unit U_Dialog_DiskEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, U_Exchange, U_HExaDeci,
  U_Dialog_FileEdit, U_Dialog_RangeEdit;

var open_diskedit_data:DB_Disk;
    open_diskedit_parent:DB_Soft;

type
  TF_DiskEdit = class(TForm)
    DI_Files: TGroupBox;
    FR_FileList: TListView;
    DI_Properties: TGroupBox;
    ST_Name: TStaticText;
    ED_Name: TEdit;
    ST_Comment: TStaticText;
    ED_Comment: TEdit;
    CB_MatchName: TCheckBox;
    FR_Delete: TButton;
    B_Cancel: TButton;
    B_Save: TButton;
    B_Okay: TButton;
    FR_Edit: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure B_CancelClick(Sender: TObject);
    procedure B_OkayClick(Sender: TObject);
    procedure B_SaveClick(Sender: TObject);
    procedure FR_EditClick(Sender: TObject);
    procedure FR_DeleteClick(Sender: TObject);
  private
    procedure ScreenToDB;
    procedure DBToScreen;
    { Private declarations }
  public
    DB:DB_Disk;
    SaveOnClose:Boolean;
    procedure RecordsToScreen;
    { Public declarations }
  end;

var
  F_DiskEdit: TF_DiskEdit;

implementation

{$R *.dfm}

procedure TF_DiskEdit.RecordsToScreen;
var TI:TListItem;
    TDF:DB_File;
    TDR:DB_Range;
begin
  TDF:=DB^.files;
  TDR:=DB^.ranges;
  FR_FileList.Clear;
  while TDF<>NIL do begin
    TI:=FR_FileList.Items.Add;
    TI.Caption:=GetPSString(TDF^.filename);
    TI.SubItems.Add(IntToStr(TDF^.size));
    TI.SubItems.Add(hexa(TDF^.crc32,8));
    TI.SubItems.Add(GetPSString(TDF^.description));
    TI.Data:=TDF;
    TDF:=TDF^.next;
  end;
  while TDR<>NIL do begin
    TI:=FR_FileList.Items.Add;
    TI.Caption:='@'+hexa(TDR^.addr1,8)+'-'+hexa(TDR^.addr2,8);
    TI.SubItems.Add(IntToStr(TDR^.addr2-TDR^.addr1+1));
    TI.SubItems.Add(hexa(TDR^.crc32,8));
    TI.SubItems.Add(GetPSString(TDR^.description));
    TI.Data:=TDR;
    TDR:=TDR^.next;
  end;
  FR_FileList.AlphaSort;
end;

procedure TF_DiskEdit.DBToScreen;
begin
  ED_Name.Text:=GetPSString(DB^.diskname);
  ED_Comment.Text:=GetPSString(DB^.comment);
  CB_MatchName.Checked:=DB^.namematch=1;
end;

procedure TF_DiskEdit.ScreenToDB;
begin
  SetPSString(DB^.diskname,ED_Name.Text);
  SetPSString(DB^.comment,ED_Comment.Text);
  if CB_MatchName.Checked then DB^.namematch:=1 else DB^.namematch:=0;
end;

procedure TF_DiskEdit.FormCreate(Sender: TObject);
var n:byte;
begin
  SaveOnClose:=TRUE;
  DB:=open_diskedit_data;
  DBToScreen;
  RecordsToScreen;
  n:=0;
  while L_DiskEdit[n].inst<>NIL do inc(n);
  L_DiskEdit[n].db:=DB;
  L_DiskEdit[n].inst:=self;
  L_DiskEdit[n].ready:=FALSE;
  L_DiskEdit[n].parent:=open_diskedit_parent;
end;

procedure TF_DiskEdit.FormClose(Sender: TObject; var Action: TCloseAction);
var n:byte;
begin
  if SaveOnClose then ScreenToDB;
  n:=0;
  while L_DiskEdit[n].inst<>self do inc(n);
  L_DiskEdit[n].ready:=TRUE;
  Release;
end;

procedure TF_DiskEdit.B_OkayClick(Sender: TObject);
begin
  SaveOnClose:=TRUE;
  Self.Close;
end;

procedure TF_DiskEdit.B_SaveClick(Sender: TObject);
begin
  ScreenToDB;
  DBToScreen;
end;

procedure TF_DiskEdit.B_CancelClick(Sender: TObject);
begin
  SaveOnClose:=FALSE;
  Self.Close;
end;

procedure TF_DiskEdit.FR_EditClick(Sender: TObject);
var c,d,e:longword;
    TDF,lastTDF:DB_File;
    TDR,lastTDR:DB_Range;
    TRE:TF_RangeEdit;
    TFE:TF_FileEdit;
begin
  c:=0; d:=0;
  fillchar(open_fileedit_pat,sizeof(open_fileedit_pat),0);
  TDF:=DB^.files;
  while TDF<>NIL do begin
    e:=0;
    while (e<FR_FileList.Items.Count) and (FR_FileList.Items.Item[e].Data<>TDF) do inc(e);
    if e<FR_FileList.Items.Count then begin
      if FR_FileList.Items.Item[e].Selected then begin
        open_fileedit_pat[e]:=TRUE;
        lastTDF:=TDF;
        inc(d);
      end;
    end;
    TDF:=TDF^.next;
    inc(c);
  end;
  if d>0 then begin
    if d>1 then begin
      open_fileedit_multi:=TRUE;
      open_fileedit_data:=DB;
    end else begin
      open_fileedit_multi:=FALSE;
      open_fileedit_data:=lastTDF;
    end;
    TFE:=TF_FileEdit.Create(self);
    TFE.Show;
  end;

  c:=0; d:=0;
  fillchar(open_rangeedit_pat,sizeof(open_rangeedit_pat),0);
  TDr:=DB^.ranges;
  while TDR<>NIL do begin
    e:=0;
    while (e<FR_FileList.Items.Count) and (FR_FileList.Items.Item[e].Data<>TDR) do inc(e);
    if e<FR_FileList.Items.Count then begin
      if FR_FileList.Items.Item[e].Selected then begin
        open_rangeedit_pat[e]:=TRUE;
        lastTDR:=TDR;
        inc(d);
      end;
    end;
    TDR:=TDR^.next;
    inc(c);
  end;
  if d>0 then begin
    if d>1 then begin
      open_rangeedit_multi:=TRUE;
      open_rangeedit_data:=DB;
    end else begin
      open_rangeedit_multi:=FALSE;
      open_rangeedit_data:=lastTDR;
    end;
    TRE:=TF_RangeEdit.Create(self);
    TRE.Show;
  end;
end;

procedure TF_DiskEdit.FR_DeleteClick(Sender: TObject);
var TDR,TTDR:DB_Range;
    TDF,TTDF:DB_File;
    c:longword;
    TI:TListItem;
begin
  for c:=0 to FR_FIleList.Items.Count-1 do begin
    TI:=FR_FileList.Items.Item[c];
    if TI.Selected then begin
      if TI.Caption[1]='@' then begin
        TDR:=TI.Data;
        if DB^.ranges=TDR then DB^.ranges:=TDR^.next else begin
          TTDR:=DB^.ranges;
          while TTDR^.next<>TDR do TTDR:=TTDR^.next;
          TTDR^.next:=TDR^.next;
        end;
        DEL_RangeContent(TDR);
        Dispose(TDR);
      end else begin
        TDF:=TI.Data;
        if DB^.files=TDF then DB^.files:=TDF^.next else begin
          TTDF:=DB^.files;
          while TTDF^.next<>TDF do TTDF:=TTDF^.next;
          TTDF^.next:=TDF^.next;
        end;
        DEL_FileContent(TDF);
        Dispose(TDF);
      end;
    end;
  end;
  RecordsToScreen;
end;

end.
