unit U_Exchange;

interface

uses StdCtrls, SysUtils, U_HexaDeci, U_ADF;

type  psstring=^shortstring;

const DBI_BOOT       =$00000001;
      DBI_PREGAME    =$00000002;
      DBI_GAME       =$00000004;
      DBI_ADDENDUM   =$00000008;
      DBI_UNUSED     =$80000000;

      DBI_CODE       =$00000010;
      DBI_SUBCODE    =$00000020;
      DBI_GRAPHICS   =$00000040;
      DBI_SOUND      =$00000080;
      DBI_CUSTOM     =$00000100;
      DBI_SAVEGAME   =$00000200;
      DBI_HIGHSCORES =$00000400;

      DBI_EXCLUDE    =$00010000;
      DBI_SHARED     =$00020000;
      DBI_OPTIONAL   =$00040000;
      DBI_WB         =$00080000;
      DBI_DYNAMIC    =$00100000;
      DBI_DYNSIZE    =$00200000;
      DBI_LOAD       =$00400000;

      DBI_RELEASETRO =$01000000;
      DBI_CRACKTRO   =$02000000;
      DBI_TRAINER    =$04000000;
      DBI_HDINSTALL  =$08000000;
      DBI_DOCS       =$10000000;

type DB_File=^_File;
       _File=record
               filename:psstring;
               size:longword;
               crc32:longword;
               info:longword;
                 { $00000001 - optional
                   $00000002 - dynamic
                   $00000004 - dynamicsize
                   $00000100 - execute
                   $00000200 - independent
                   $00000400 - addition
                   $00010000 - code
                   $00020000 - graphics
                   $00040000 - soundfx
                   $00080000 - music
                   $01000000 - highscore
                   $02000000 - savegame
                   $80000000 - exclude }
               description:psstring;
               prev:DB_File;
               next:DB_File;
               ext:pointer;
             end;

     DB_Range=^_Range;
       _Range=record
                addr1:longword;
                addr2:longword;
                crc32:longword;
                info:longword;
                description:psstring;
                prev:DB_Range;
                next:DB_Range;
                ext:pointer;
              end;

     DB_Disk=^_Disk;
       _Disk=record
               diskname:psstring;
               comment:psstring;
               namematch:byte; { 0 = off, 1 = must match }
               files:DB_File;
               ranges:DB_Range;
               next:DB_Disk;
               prev:DB_Disk;
               ext:pointer;
             end;

     DB_Version=^_Version;
       _Version=record
                  graphicschip:byte; { $00 - OCS, $01 - ECS, $02 - AGA }
                  releaseyear:word;
                  releasemonth:byte;
                  releaseday:byte;
                  gamename:psstring;
                  design:psstring;
                  publisher:psstring;
                  developer:psstring;
                  music:psstring;
                  soundfx:psstring;
                  graphics:psstring;
                  programm:psstring;
                  parent:byte;
                  system:psstring;
                  versionname:psstring;
                  comment:psstring;
                  protection:psstring;
                  cracker:psstring;
                  releaser:psstring;
                  fix:byte; { $01 - AGA, $02 - HD, $04 - 680x0 }
                  fixer:psstring;
                  train:word; { $kkoo - ingamekeys, options }
                  trainer:psstring;
                  modification:psstring;
                  modificator:psstring;
                  memc:word;
                  mems:word;
                  memf:word;
                  lang:longword; { $00000001 - En
                                   $00000002 - De
                                   $00000004 - Pl
                                   $00000008 - Dk
                                   $00000010 - Se
                                   $00000020 - No
                                   $00000040 - Nl
                                   $00000080 - Be
                                   $80000000 - Other }

                  disks:array[$00..$ff] of byte;
                  prev:DB_Version;
                  next:DB_Version;
                  ext:pointer;
                  compat:record
                           cpu:word; { $0003 - 68000
                                       $000c - 68010
                                       $0030 - 68020
                                       $00c0 - 680ec20
                                       $0300 - 68040
                                       $0c00 - FPU }
                           chipset:byte; { $0003 - cycle exact
                                           $000c - immediate blitter
                                           $0030 - fast copper }
                           floppy:word;  { $0003 - 2x
                                           $000c - 4x
                                           $0030 - 8x
                                           $00c0 - Turbo
                                           $0300 - special number of drives
                                           $x000 - number of drives }
                           graphics:byte; { $0003 - OCS
                                            $000c - ECS
                                            $0030 - AGA }
                           memory:byte; { $0003 - special settings }
                           memchip:word;
                           memslow:word;
                           memfast:word;
                           kickstart:word; { $0003 - KS12
                                             $000c - KS13
                                             $0030 - KS20
                                             $00c0 - KS30
                                             $0300 - KS31
                                             $0c00 - KS35 }
                           special:word; { $0003 - more compatible 68000 emul
                                           $000c - fastest speed
                                           $0030 - needs speed adjust
                                           $x000 - speed }
                           winfellow:longword; { $yymmdd03 - tested }
                           winuae:longword;    { $xyyzzr03 - tested }
                         end;
                end;

      DB_Soft=^_Soft;
        _Soft=record
                gamename:psstring;
                releaseyear:word;
                releasemonth:byte;
                releaseday:byte;
                publisher:psstring;
                developer:psstring;
                music:psstring;
                soundfx:psstring;
                graphics:psstring;
                programm:psstring;
                design:psstring;
                comment:psstring;
                versions:DB_Version;
                disks:DB_Disk;
                prev:DB_Soft;
                next:DB_Soft;
                ext:pointer;
              end;

      DB_Base=^_Base;
        _Base=record
                name:psstring;
                creator:psstring;
                releaseyear:word;
                releasemonth:byte;
                releaseday:byte;
                softs:DB_Soft;
                next:DB_Base;
                prev:DB_Base;
                ext:pointer;
              end;

{ SCAN TYPE }

type DS_Status=record
                 pmx,pbx,pdx,pnx:longword;
                 mmx,mbx,mdx,mnx:longword;
                 amx,abx,adx,anx:longword;
                 umx,ubx,udx,unx:longword;
                 xx:longword;
                 nb,nd,tu,cu,ru,hu,du,sd,hd:boolean;
                 sdx,hdx:longword;
                 sgx,sgy:longword;
               end;

type SC_File=^SCFile;
      SCFile=record
               DBF:DB_File;
               DBD:DB_Disk;
               DBS:DB_Soft;
               SCL:longword;
               SCR:longword;
               SCE:longword;
               ct:longword;
             end;

type SC_Range=^SCRange;
      SCRange=record
                DBR:DB_Range;
                DBD:DB_Disk;
                DBS:DB_Soft;
                SCL:longword;
                SCR:longword;
                SCE:longword;
                ct:longword;
              end;

type SCScanF=array[0..0] of SCFile;
     SCScanR=array[0..0] of SCRange;
     SC_Scan=record
               ctf:longword;
               ctr:longword;
               scf:^SCScanF;
               scr:^SCScanR;
             end;
     SC_Result=^SC_Res;
     SC_Res=record
              DBD:DB_Disk;
              DBS:DB_Soft;
              ACC:LongWord;
              DS:DS_Status;
              next:SC_Result;
            end;


type DBE_Range=^DBEC_Range;
     DBEC_Range=record
                  ID:longword;
                  DID:longword;
                  Name:string[$77];
                  Desc:string[$73];
                  RBeg:longword;
                  REnd:longword;
                  Size:longword;
                  CRC32:longword;
                  Attr:longword;
                  Pack:word;
                  Prev:DBE_Range;
                  Next:DBE_Range;
                end;

type DBE_File=^DBEC_File;
     DBEC_File=record
                 ID:longword;
                 DID:longword;
                 Name:string[$77];
                 Desc:string[$73];
                 Size:longword;
                 CRC32:longword;
                 LastError:longword;
                 FHBN:longword;
                 Attr:longword;
                    { $0000 = main
                      $0001 = code
                      $0002 = custom data
                      $0004 = sound
                      $0008 = graphics
                      $0010 = savegame
                      $0020 = highscore
                      $0040 = custom info
                      $0080 = bootcode (NDOS only)
                      $0100 = mandatory
                      $0200 = dynamic
                      $0400 = addon
                      $0800 = packed with known packer }
                 Pack:word;
                 Prev:DBE_File;
                 Next:DBE_File;
               end;

var L_Inspector:array[$00..$ff] of pointer;
    L_Hex:array[$00..$ff] of record
                               bufp:pointer;
                               bufl:longword;
                               ready:boolean;
                             end;
    L_FileEdit:array[$00..$ff] of record
                                   db:DB_File;
                                   inst:pointer;
                                   ready:boolean;
                                 end;
    L_RangeEdit:array[$00..$ff] of record
                                    db:DB_Range;
                                    inst:pointer;
                                    ready:boolean;
                                   end;
    L_DiskEdit:array[$00..$ff] of record
                                   parent:DB_Soft;
                                   db:DB_Disk;
                                   inst:pointer;
                                   ready:boolean;
                                 end;
    L_VersionEdit:array[$00..$ff] of record
                                      db:DB_Version;
                                      inst:pointer;
                                      ready:boolean;
                                     end;
    L_GameEdit:array[$00..$ff] of record
                                    db:DB_Soft;
                                    inst:pointer;
                                    ready:boolean;
                                   end;
    L_DataBaseEdit:array[$00..$ff] of record
                                       db:DB_Base;
                                       inst:pointer;
                                       ready:boolean;
                                      end;


procedure DEL_FileContent(var DBF:DB_File);
procedure DEL_File(var DBF:DB_File);
procedure DEL_RangeContent(var DBR:DB_Range);
procedure DEL_Range(var DBR:DB_Range);
procedure DEL_DiskContent(var DBD:DB_Disk);
procedure DEL_Disk(var DBD:DB_Disk);
procedure DEL_VersionContent(var DBV:DB_Version);
procedure DEL_Version(var DBV:DB_Version);
procedure DEL_SoftContent(var DBS:DB_Soft);
procedure DEL_Soft(var DBS:DB_Soft);
procedure DEL_BaseContent(var DBB:DB_Base);
procedure DEL_Base(var DBB:DB_Base);
procedure DelPSString(var p:psstring);
procedure SetPSString(var p:psstring; s:string);
function  GetPSString(var p:psstring):string;
function  MemoToString(TM:TMemo):string;
function  UpcaseStr(s:string):string;
procedure DIDSave(DBB:DB_Base; fn:string);
procedure DIDLoad(var DBB:DB_Base; fn:string);
function SCN_HigherF(DBF1,DBF2:DB_File):boolean;
function SCN_HigherR(DBR1,DBR2:DB_Range):boolean;
procedure SCN_InitDB(DB:DB_Base; var SC:SC_Scan);
procedure SCN_InitSC(var SC:SC_Scan);
procedure SCN_DeInit(var SC:SC_Scan);
function GetTagName(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagVersion(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagDate(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagPublisher(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagProtection(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagReleased(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagCracked(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagTrained(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagFix(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagModified(DBS:DB_Soft; DBV:DB_Version):string;
function GetTagLanguage(DBS:DB_Soft; DBV:DB_Version):string;
function GetVersionNameEx(DBS:DB_Soft; DBV:DB_Version):string;
function GetDiskName(DBS:DB_Soft; DBD:DB_Disk):string;
function StringsMatch(s1,s2:string):boolean;

implementation

function StringsMatch(s1,s2:string):boolean;
var okay:boolean;
    b1,b2,bh:word;
begin
  b1:=0;
  b2:=0;
  s1:=UpcaseStr(s1);
  s2:=UpcaseStr(s2);
  okay:=TRUE;
  while okay and (b1<=length(s1)) and (b2<=length(s2)) do begin
    if (s1[b1]=s2[b2]) or (s1[b1]='?') then begin inc(b1); inc(b2); end else
    if (b1=length(s1)) and (s1[b1]='*') then begin inc(b1); b2:=length(s2)+1; end else
    if s1[b1]='*' then begin
      bh:=b2;
      while (bh<=length(s2)) and (not StringsMatch( copy(s1,b1+1,length(s1)-b1), copy(s2,bh,length(s2)-bh+1) )) do begin
        inc(bh);
        while (s1[b1+1]<>s2[bh]) and (bh<=length(s2)) do inc(bh);
      end;
      if bh>length(s2) then okay:=FALSE;
      b1:=length(s1)+1;
      b2:=length(s2)+1;
    end else okay:=FALSE;
  end;
  StringsMatch:=okay and (b1=length(s1)+1) and (b2=length(s2)+1);
end;

function GetDiskName(DBS:DB_Soft; DBD:DB_Disk):string;
var TDBF:DB_File;
    TDBEF:DBE_File;
    filefound:boolean;
    files,filesfound:longword;
    diskname:string;
    disknum,diskmax:longword;
    DBVL:array[$00..$ff] of DB_Version;
    DBVN:longword;
    DBVU:array[$00..$ff] of DB_Version;
    DBVM:longword;
    DBDI:longword;
    TDBD:DB_Disk;
    TDBV:DB_Version;
    l0,l1,l2,l3:longint;
    s,d1,d2:string;  { %d1(Dxy)%d2 }
    tagsbehind,taguse,tagdifferent:boolean;
    multiversion:boolean;
begin
 { get disk number }
  TDBD:=DBS^.disks; DBDI:=1;
  while TDBD<>DBD do begin inc(DBDI); TDBD:=TDBD^.next; end;

 { get possible versions }
  TDBV:=DBS^.versions; DBVN:=0;
  while TDBV<>NIL do begin
    l0:=0;
    while TDBV^.disks[l0]<>0 do begin
      if TDBV^.disks[l0]=DBDI then begin DBVL[DBVN]:=TDBV; inc(DBVN); end;
      inc(l0);
    end;
    TDBV:=TDBV^.next;
  end;
  multiversion:=DBVN>1;

if DBVN>0 then begin

 { get disk share status => tags behind? }
  tagsbehind:=FALSE; DBVM:=0;
  fillchar(DBVU,sizeof(DBVU),0);  { clear list of versions which share disks }
  for l0:=$00 to $ff do begin     { for every disk of found version }
    l1:=DBVL[0].disks[l0];        { check if disk occurs on other version too }
    if l1<>0 then begin
      TDBV:=DBS^.versions;
      while TDBV<>NIL do begin
        if TDBV<>DBVL[0] then begin  { if comared version aren't identical }
          for l2:=$00 to $ff do begin
            if TDBV^.disks[l2]=l1 then begin   { if used disk is equal }
              l3:=0;
              while (DBVU[l3]<>TDBV) and (l3<DBVM) do inc(l3);
              if l3=DBVM then begin      { add version to shared lsit }
                DBVU[l3]:=TDBV;
                inc(DBVM);
              end;
            end;
          end;
        end;
        TDBV:=TDBV^.next;
      end;
    end;
  end;
  tagsbehind:=DBVM>0;

  DiskName:='';
  if multiversion then DiskName:=DiskName+'[mv]';
  if tagsbehind then DiskName:=DiskName+'[tb]';

  d1:=GetTagName(DBS,DBVL[0])
     +GetTagVersion(DBS,DBVL[0])
     +GetTagDate(DBS,DBVL[0])
     +GetTagPublisher(DBS,DBVL[0]);
  d2:='';

  s:=GetTagLanguage(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagLanguage(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagLanguage(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagProtection(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagProtection(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagProtection(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagReleased(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagReleased(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagReleased(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagCracked(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagCracked(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagCracked(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagTrained(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagTrained(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagTrained(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagFix(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagFix(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagFix(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

  s:=GetTagModified(DBS,DBVL[0]); taguse:=TRUE; tagdifferent:=FALSE;
  if multiversion then for l0:=1 to DBVN-1 do if s<>GetTagModified(DBS,DBVL[l0]) then taguse:=FALSE;
  if tagsbehind then for l0:=0 to DBVM-1 do if s<>GetTagModified(DBS,DBVU[l0]) then tagdifferent:=TRUE;
  if taguse then begin if tagdifferent then d2:=d2+s else d1:=d1+s; end;

 { attach media info }
  l0:=0; while DBVL[0]^.disks[l0]<>DBDI do inc(l0);
  l1:=l0; while DBVL[0]^.disks[l1]<>0 do inc(l1); inc(l0);
  if l1<>1 then begin
    if l1<10 then d1:=d1+'['+IntToStr(l0)+'|'+IntToStr(l1)+']' else begin
      if l0<10 then d1:=d1+'[0'+IntToStr(l0)
               else d1:=d1+'['+IntToStr(l0);
      d1:=d1+'-'+IntToStr(l1)+']';
    end;
  end;

  if MultiVersion then GetDiskName:=d1+'[mv]'+d2 else GetDiskName:=d1+d2;
end else GetDiskName:=GetPSString(DBD^.diskname);
end;

function GetVersionNameEx(DBS:DB_Soft; DBV:DB_Version):string;
var s:string;
begin
  s:=GetTagLanguage(DBS,DBV)
    +GetTagProtection(DBS,DBV)
    +GetTagCracked(DBS,DBV)
    +GetTagTrained(DBS,DBV)
    +GetTagFix(DBS,DBV)
    +GetTagModified(DBS,DBV);
  GetVersionNameEx:=s;
end;

function GetTagName(DBS:DB_Soft; DBV:DB_Version):string;
{ GameName (OriginalGameName) AGA }
var s,s1,s2,s3:string;
begin
  s1:=GetPSSTring(DBS^.gamename);
  s2:=GetPSSTring(DBV^.gamename);
  if (s2='') or (s2=s1) then s:=s1 else s:=s2;
  GetTagName:=s+' ';
end;

function GetTagVersion(DBS:DB_Soft; DBV:DB_Version):string;
var s,s1:string;
begin
  if DBV^.graphicschip=2 then s:='AGA ' else s:='';
  s1:=GetPSString(DBV^.versionname);
  if s1<>'' then s:=s+s1+' ';
  GetTagVersion:=s;
end;

function GetTagDate(DBS:DB_Soft; DBV:DB_Version):string;
{ (1999-07-27) }
var s:string;
begin
  if DBV^.releaseyear<>0 then begin
    s:=IntToStr(DBV^.releaseyear);
    if DBV^.releasemonth<>0 then begin
      s:=s+'-'; if DBV^.releasemonth<10 then s:=s+'0';
      s:=s+IntToStr(DBV^.releasemonth);
      if DBV^.releaseday<>0 then begin
        s:=s+'-'; if DBV^.releaseday<10 then s:=s+'0';
        s:=s+IntToStr(DBV^.releaseday);
      end;
    end;
  end else
  if DBS^.releaseyear<>0 then begin
    s:=IntToStr(DBS^.releaseyear);
    if DBS^.releasemonth<>0 then begin
      s:=s+'-'; if DBS^.releasemonth<10 then s:=s+'0';
      s:=s+IntToStr(DBS^.releasemonth);
      if DBS^.releaseday<>0 then begin
        s:=s+'-'; if DBS^.releaseday<10 then s:=s+'0';
        s:=s+IntToStr(DBS^.releaseday);
      end;
    end;
  end else s:='-';
  GetTagDate:='('+s+')';
end;

function GetTagPublisher(DBS:DB_Soft; DBV:DB_Version):string;
{ (PublisherName) }
var s,s1,s2:string;
begin
  s1:=GetPSString(DBS^.publisher);
  s2:=GetPSString(DBV^.publisher);
  if (s2='') or (s2=s1) then s:=s1 else s:=s2;
  GetTagPublisher:='('+s+')';
end;

function GetTagProtection(DBS:DB_Soft; DBV:DB_Version):string;
{ [cp Protection] }
var s1,s2:string;
begin
  s1:=GetPSString(DBV^.protection);
  s2:=GetPSString(DBV^.cracker);
  if (s1<>'') and (s2='') then GetTagProtection:='[cp '+s1+']' else GetTagProtection:='';
end;

function GetTagCracked(DBS:DB_Soft; DBV:DB_Version):string;
{ [cr Cracker] }
var s:string;
begin
  s:=GetPSString(DBV^.cracker);
  if s<>'' then GetTagCracked:='[cr '+s+']' else GetTagCracked:='';
end;

function GetTagReleased(DBS:DB_Soft; DBV:DB_Version):string;
{ [r Releaser] }
var s:string;
begin
  s:=GetPSString(DBV^.releaser);
  if s<>'' then GetTagReleased:='[r '+s+']' else GetTagReleased:='';
end;

function GetTagTrained(DBS:DB_Soft; DBV:DB_Version):string;
{ [t +3(7) Anthrox] }
var s,s1,s2:string;
begin
  s1:=GetPSString(DBV^.trainer);
  if (DBV^.train<>0) or (s1<>'') then begin
    s:='[t +';
    if DBV^.train>$ff then begin
      s:=s+IntToStr(DBV^.train and $ff+1)+'('+IntToStr(DBV^.train shr 8)+')';
    end else s:=s+IntToStr(DBV^.train);
    if s1<>'' then s:=s+' '+s1;
    s:=s+']';
  end else s:='';
  GetTagTrained:=s;
end;

function GetTagFix(DBS:DB_Soft; DBV:DB_Version):string;
{ [f AGA-HD-680x0 Fixer] }
var s,s1,s2,s3:string;
begin
  s1:=GetPSString(DBV^.Fixer);
  if (DBV^.Fix<>0) or (s1<>'') then begin
    if DBV^.Fix=0 then s:=s1 else begin
      if DBV^.fix and $01=$01 then begin if s<>'' then s:=s+'-'; s:=s+'AGA'; end;
      if DBV^.fix and $02=$02 then begin if s<>'' then s:=s+'-'; s:=s+'HD'; end;
      if DBV^.fix and $04=$04 then begin if s<>'' then s:=s+'-'; s:=s+'680x0'; end;
      if s1<>'' then s:=s+' '+s1;
    end;
    s:='[f '+s+']';
  end else s:='';
  GetTagFix:=s;
end;

function GetTagModified(DBS:DB_Soft; DBV:DB_Version):string;
{ [m Type Modificator] }
var s,s1,s2:string;
begin
  s1:=GetPSString(DBV^.modification);
  s2:=GetPSString(DBV^.modificator);
  if (s1<>'') or (s2<>'') then begin
    if s2='' then s:=s1 else begin
      if s1='' then s:=s2 else begin
        s:=s1+' '+s2;
      end;
    end;
    s:='[m '+s+']';
  end else s:='';
  GetTagModified:=s;
end;

function GetTagLanguage(DBS:DB_Soft; DBV:DB_Version):string;
{ (En-De-Pl-XX) }
var s:string;
begin
  s:='';
  if DBV^.lang<>$01 then begin
    if DBV^.lang and $01=$01 then begin if s<>'' then s:=s+'-'; s:=s+'En'; end;
    if DBV^.lang and $02=$02 then begin if s<>'' then s:=s+'-'; s:=s+'De'; end;
    if DBV^.lang and $04=$04 then begin if s<>'' then s:=s+'-'; s:=s+'Pl'; end;
    if DBV^.lang and $08=$08 then begin if s<>'' then s:=s+'-'; s:=s+'Dk'; end;
    if DBV^.lang and $10=$10 then begin if s<>'' then s:=s+'-'; s:=s+'Se'; end;
    if DBV^.lang and $20=$20 then begin if s<>'' then s:=s+'-'; s:=s+'No'; end;
    if DBV^.lang and $40=$40 then begin if s<>'' then s:=s+'-'; s:=s+'Nl'; end;
    if DBV^.lang and $80=$80 then begin if s<>'' then s:=s+'-'; s:=s+'Fr'; end;
    if DBV^.lang shr $1f=$01 then begin if s<>'' then s:=s+'-'; s:=s+'xx'; end;
    s:='('+s+')';
  end else s:='';
  GetTagLanguage:=s;
end;

function SCN_HigherF(DBF1,DBF2:DB_File):boolean;
var hi:byte;
begin
  if (DBF1.crc32=DBF2.crc32) and (DBF1.size=DBF2.size) then hi:=1 else
  if (DBF1.crc32=DBF2.crc32) and (DBF1.size>DBF2.size) then hi:=0 else
  if (DBF1.crc32=DBF2.crc32) and (DBF1.size<DBF2.size) then hi:=2 else
  if (DBF1.crc32>DBF2.crc32) then hi:=0 else
  if (DBF1.crc32<DBF2.crc32) then hi:=2;
  SCN_HigherF:=hi=0;
end;

function SCN_HigherR(DBR1,DBR2:DB_Range):boolean;
var hi:byte;
begin
  if (DBR1.crc32=DBR2.crc32) and (DBR1.addr1=DBR2.addr1) then hi:=1 else
  if (DBR1.crc32=DBR2.crc32) and (DBR1.addr1>DBR2.addr1) then hi:=0 else
  if (DBR1.crc32=DBR2.crc32) and (DBR1.addr1<DBR2.addr1) then hi:=2 else
  if (DBR1.crc32>DBR2.crc32) then hi:=0 else
  if (DBR1.crc32<DBR2.crc32) then hi:=2;
  SCN_HigherR:=hi=0;
end;

procedure SCN_InitDB(DB:DB_Base; var SC:SC_Scan);
var TDS:DB_Soft;
    TDD:DB_Disk;
    TDF:DB_File;
    t:text;
    TDR:DB_Range;
    p:longword;
begin
  SC.ctf:=0;
  SC.ctr:=0;
  TDS:=DB^.softs;
  while TDS<>NIL do begin
    TDD:=TDS^.disks;
    while TDD<>NIL do begin
      TDF:=TDD^.files;
      while TDF<>NIL do begin
       { xx }
        if TDF^.info and DBI_EXCLUDE=$00000000 then inc(SC.ctf);
        TDF:=TDF^.next;
      end;
      TDR:=TDD^.ranges;
      while TDR<>NIL do begin
       { xx }
        if (TDR^.info and DBI_EXCLUDE=$00000000) and (TDR^.addr2-TDR^.addr1=$1ff) then inc(SC.ctr);
        TDR:=TDR^.next;
      end;
      TDD:=TDD^.next;
    end;
    TDS:=TDS^.next;
  end;
  getmem(SC.scf,sizeof(SCFIle)*SC.ctf);
  getmem(SC.scr,sizeof(SCRange)*SC.ctr);
  fillchar(SC.scf^,sizeof(SCFIle)*SC.ctf,0);
  fillchar(SC.scr^,sizeof(SCRange)*SC.ctr,0);

  SC.ctf:=0;
  SC.ctr:=0;
  TDS:=DB^.softs;
  while TDS<>NIL do begin
    TDD:=TDS^.disks;
    while TDD<>NIL do begin
      TDF:=TDD^.files;
      while TDF<>NIL do begin
        if (TDF^.info and DBI_EXCLUDE=$00000000) then begin
          SC.scf[sc.ctf].DBF:=TDF;
          SC.scf[sc.ctf].DBD:=TDD;
          SC.scf[sc.ctf].DBS:=TDS;
          if SC.ctf<>0 then begin
            p:=0;
            repeat
              if SCN_HigherF(SC.scf[p].DBF,TDF) then begin
                if SC.scf[p].SCL=0 then begin
                  SC.scf[p].SCL:=sc.ctf;
                  p:=0;
                end else begin
                  p:=SC.scf[p].SCL;
                end;
              end else if SCN_HigherF(TDF,SC.scf[p].DBF) then begin
                if SC.scf[p].SCR=0 then begin
                  SC.scf[p].SCR:=sc.ctf;
                  p:=0;
                end else begin
                  p:=SC.scf[p].SCR;
                end;
              end else begin
                while SC.scf[p].SCE<>0 do p:=SC.scf[p].SCE;
                SC.scf[p].SCE:=sc.ctf;
                p:=0;
              end;
            until p=0;
          end;
          inc(SC.ctf);
        end;
        TDF:=TDF^.next;
      end;
      TDR:=TDD^.ranges;
      while TDR<>NIL do begin
        if (TDR^.info and DBI_EXCLUDE=$00000000) and (TDR^.addr2-TDR^.addr1=$1ff) then begin
          SC.scr[sc.ctr].DBR:=TDR;
          SC.scr[sc.ctr].DBD:=TDD;
          SC.scr[sc.ctr].DBS:=TDS;
          if SC.ctr<>0 then begin
            p:=0;
            repeat
              if SCN_HigherR(SC.scr[p].DBR,TDR) then begin
                if SC.scr[p].SCL=0 then begin
                  SC.scr[p].SCL:=sc.ctr;
                  p:=0;
                end else begin
                  p:=SC.scr[p].SCL;
                end;
              end else if SCN_HigherR(TDR,SC.scr[p].DBR) then begin
                if SC.scr[p].SCR=0 then begin
                  SC.scr[p].SCR:=sc.ctr;
                  p:=0;
                end else begin
                  p:=SC.scr[p].SCR;
                end;
              end else begin
                while SC.scr[p].SCE<>0 do p:=SC.scr[p].SCE;
                SC.scr[p].SCE:=sc.ctr;
                p:=0;
              end;
            until p=0;
          end;
          inc(SC.ctr);
        end;
        TDR:=TDR^.next;
      end;
      TDD:=TDD^.next;
    end;
    TDS:=TDS^.next;
  end;
end;

procedure SCN_InitSC(var SC:SC_Scan);
var p:longword;
begin
  for p:=0 to SC.ctf-1 do SC.scf^[p].SCL:=0;
  for p:=0 to SC.ctf-1 do SC.scf^[p].SCR:=0;
  for p:=0 to SC.ctr-1 do SC.scr^[p].SCL:=0;
  for p:=0 to SC.ctr-1 do SC.scr^[p].SCR:=0;
end;

procedure SCN_DeInit(var SC:SC_Scan);
begin
  freemem(SC.scf,sizeof(SCFile)*SC.ctf);
  freemem(SC.scr,sizeof(SCRange)*SC.ctr);
end;

procedure DEL_FileContent(var DBF:DB_File);
begin
  if DBF^.filename<>NIL then Freemem(DBF^.filename,length(DBF^.filename^)+1);
  if DBF^.description<>NIL then Freemem(DBF^.description,length(DBF^.description^)+1);
end;

procedure DEL_File(var DBF:DB_File);
var TDBF,TMP:DB_File;
begin
  TDBF:=DBF;
  while TDBF<>NIL do begin
    DEL_FileContent(TDBF);
    TMP:=TDBF^.next;
    Dispose(TDBF);
    TDBF:=TMP;
  end;
  DBF:=NIL;
end;

procedure DEL_RangeContent(var DBR:DB_Range);
begin
  if DBR^.description<>NIL then Freemem(DBR^.description,length(DBR^.description^)+1);
end;

procedure DEL_Range(var DBR:DB_Range);
var TDBR,TMP:DB_Range;
begin
  TDBR:=DBR;
  while TDBR<>NIL do begin
    DEL_RangeContent(TDBR);
    TMP:=TDBR^.next;
    Dispose(TDBR);
    TDBR:=TMP;
  end;
  DBR:=NIL;
end;

procedure DEL_DiskContent(var DBD:DB_Disk);
begin
  if DBD^.diskname<>NIL then Freemem(DBD^.diskname,length(DBD^.diskname^)+1);
  if DBD^.comment<>NIL then Freemem(DBD^.comment,length(DBD^.comment^)+1);
end;

procedure DEL_Disk(var DBD:DB_Disk);
var TDBD,TMP:DB_Disk;
begin
  TDBD:=DBD;
  while TDBD<>NIL do begin
    DEL_File(TDBD^.Files);
    DEL_Range(TDBD^.Ranges);
    DEL_DiskContent(TDBD);
    TMP:=TDBD^.next;
    Dispose(TDBD);
    TDBD:=TMP;
  end;
  DBD:=NIL;
end;

procedure DEL_VersionContent(var DBV:DB_Version);
begin
  if DBV^.publisher<>NIL then Freemem(DBV^.publisher,length(DBV^.publisher^)+1);
  if DBV^.developer<>NIL then Freemem(DBV^.developer,length(DBV^.developer^)+1);
  if DBV^.music<>NIL then Freemem(DBV^.music,length(DBV^.music^)+1);
  if DBV^.soundfx<>NIL then Freemem(DBV^.soundfx,length(DBV^.soundfx^)+1);
  if DBV^.graphics<>NIL then Freemem(DBV^.graphics,length(DBV^.graphics^)+1);
  if DBV^.programm<>NIL then Freemem(DBV^.programm,length(DBV^.programm^)+1);
  if DBV^.system<>NIL then Freemem(DBV^.system,length(DBV^.system^)+1);
  if DBV^.versionname<>NIL then Freemem(DBV^.versionname,length(DBV^.versionname^)+1);
  if DBV^.comment<>NIL then Freemem(DBV^.comment,length(DBV^.comment^)+1);
  if DBV^.protection<>NIL then Freemem(DBV^.protection,length(DBV^.protection^)+1);
  if DBV^.cracker<>NIL then Freemem(DBV^.cracker,length(DBV^.cracker^)+1);
  if DBV^.fixer<>NIL then Freemem(DBV^.fixer,length(DBV^.fixer^)+1);
  if DBV^.trainer<>NIL then Freemem(DBV^.trainer,length(DBV^.trainer^)+1);
  if DBV^.modification<>NIL then Freemem(DBV^.modification,length(DBV^.modification^)+1);
  if DBV^.modificator<>NIL then Freemem(DBV^.modificator,length(DBV^.modificator^)+1);
end;

procedure DEL_Version(var DBV:DB_Version);
var TDBV,TMP:DB_Version;
begin
  TDBV:=DBV;
  while TDBV<>NIL do begin
    DEL_VersionContent(TDBV);
    TMP:=TDBV^.next;
    Dispose(TDBV);
    TDBV:=TMP;
  end;
  DBV:=NIL;
end;

procedure DEL_SoftContent(var DBS:DB_Soft);
begin
  if DBS^.gamename<>NIL then Freemem(DBS^.gamename,length(DBS^.gamename^)+1);
  if DBS^.publisher<>NIL then Freemem(DBS^.publisher,length(DBS^.publisher^)+1);
  if DBS^.developer<>NIL then Freemem(DBS^.developer,length(DBS^.developer^)+1);
  if DBS^.music<>NIL then Freemem(DBS^.music,length(DBS^.music^)+1);
  if DBS^.soundfx<>NIL then Freemem(DBS^.soundfx,length(DBS^.soundfx^)+1);
  if DBS^.graphics<>NIL then Freemem(DBS^.graphics,length(DBS^.graphics^)+1);
  if DBS^.programm<>NIL then Freemem(DBS^.programm,length(DBS^.programm^)+1);
  if DBS^.comment<>NIL then Freemem(DBS^.comment,length(DBS^.comment^)+1);
  if DBS^.design<>NIL then Freemem(DBS^.design,length(DBS^.design^)+1);
end;

procedure DEL_Soft(var DBS:DB_Soft);
var TDBS,TMP:DB_Soft;
begin
  TDBS:=DBS;
  while TDBS<>NIL do begin
    DEL_Disk(TDBS^.Disks);
    DEL_Version(TDBS^.Versions);
    DEL_SoftContent(TDBS);
    TMP:=TDBS^.next;
    Dispose(TDBS);
    TDBS:=TMP;
  end;
  DBS:=NIL;
end;

procedure DEL_BaseContent(var DBB:DB_Base);
begin
  if DBB^.name<>NIL then Freemem(DBB^.name,length(DBB^.name^)+1);
  if DBB^.creator<>NIL then Freemem(DBB^.creator,length(DBB^.creator^)+1);
end;

procedure DEL_Base(var DBB:DB_Base);
var TDBB,TMP:DB_Base;
begin
  TDBB:=DBB;
  while TDBB<>NIL do begin
    DEL_BaseContent(TDBB);
    DEL_Soft(TDBB.Softs);
    TMP:=TDBB^.next;
    Dispose(TDBB);
    TDBB:=TMP;
  end;
  DBB:=NIL;
end;

procedure DelPSString(var p:psstring);
begin
  if p<>NIL then freemem(p,length(p^)+1);
  p:=NIL;
end;

procedure SetPSString(var p:psstring; s:string);
begin
  DelPSString(p);
  if s<>'' then begin
    getmem(p,length(s)+1);
    p^:=s;
  end;
end;

function GetPSString(var p:psstring):string;
begin
  if p<>NIL then GetPSString:=p^ else GetPSString:='';
end;

function MemoToString(TM:TMemo):string;
var ln:longword;
    s:string;
begin
  s:='';
  if TM.Lines.Count>0 then for ln:=0 to TM.Lines.Count-1 do begin
    if s<>'' then s:=s+' ';
    s:=s+TM.Lines.Strings[ln];
  end;
  MemoToString:=s;
end;

function UpcaseStr(s:string):string;
var l:longword;
begin
  for l:=1 to length(s) do s[l]:=upcase(s[l]);
  UpcaseStr:=s;
end;

function Sp(n:longword):string;
var s:string;
begin
  s:='';
  while n>0 do begin s:=s+' '; dec(n); end;
  Sp:=s;
end;

procedure DIDSave(DBB:DB_Base; fn:string);
var t:text;
    TDB:DB_Base;
    TDS:DB_Soft;
    TDV:DB_Version;
    TDD:DB_Disk;
    TDF:DB_File;
    TDR:DB_Range;
    n,m:longword;
begin
  assign(t,fn);
  rewrite(t);

 { databse info }
  TDB:=DBB;
  writeln(t,'info ( name "',GetPSString(TDB^.name),'" creator "',GetPSString(TDB^.creator),'" date ',
          TDB^.releaseyear,TDB^.releasemonth div 10,TDB^.releasemonth mod 10,TDB^.releaseday div 10,TDB^.releaseday mod 10,' )');

 { software }
  TDS:=TDB^.softs;
  while TDS<>NIL do begin
    writeln(t,'soft (');
    if TDS^.gamename<>NIL then  writeln(t,'name "'+GetPSString(TDS^.gamename),'"');
    if TDS^.releaseyear<>0 then writeln(t,'rdat ',TDS^.releaseyear,
                           TDS^.releasemonth div 10,TDS^.releasemonth mod 10,
                           TDS^.releaseday div 10,TDS^.releaseday mod 10);
    if TDS^.publisher<>NIL then writeln(t,'publ "'+GetPSString(TDS^.publisher),'"');
    if TDS^.developer<>NIL then writeln(t,'devl "'+GetPSString(TDS^.developer),'"');
    if TDS^.programm<>NIL then  writeln(t,'code "'+GetPSString(TDS^.programm),'"');
    if TDS^.graphics<>NIL then  writeln(t,'grfx "'+GetPSString(TDS^.graphics),'"');
    if TDS^.music<>NIL then     writeln(t,'musi "'+GetPSString(TDS^.music),'"');
    if TDS^.soundfx<>NIL then   writeln(t,'sdfx "'+GetPSString(TDS^.soundfx),'"');
    if TDS^.design<>NIL then    writeln(t,'dsgn "'+GetPSString(TDS^.design),'"');
    if TDS^.comment<>NIL then   writeln(t,'cmmt "'+GetPSString(TDS^.comment),'"');

   { versions }
    TDV:=TDS^.versions;
    while TDV<>NIL do begin
      writeln(t,'version ( ');
      if TDV^.gamename<>NIL then  writeln(t,'gnam "'+GetPSString(TDV^.gamename),'"');
      if TDV^.releaseyear<>0 then writeln(t,'rdat ',TDV^.releaseyear,
                             TDV^.releasemonth div 10,TDV^.releasemonth mod 10,
                             TDV^.releaseday div 10,TDV^.releaseday mod 10);
      if TDV^.publisher<>NIL then writeln(t,'publ "'+GetPSString(TDV^.publisher),'"');
      if TDV^.developer<>NIL then writeln(t,'devl "'+GetPSString(TDV^.developer),'"');
      if TDV^.programm<>NIL then  writeln(t,'code "'+GetPSString(TDV^.programm),'"');
      if TDV^.graphics<>NIL then  writeln(t,'grfx "'+GetPSString(TDV^.graphics),'"');
      if TDV^.music<>NIL then     writeln(t,'musi "'+GetPSString(TDV^.music),'"');
      if TDV^.soundfx<>NIL then   writeln(t,'sdfx "'+GetPSString(TDV^.soundfx),'"');
      if TDV^.design<>NIL then    writeln(t,'dsgn "'+GetPSString(TDV^.design),'"');
      if TDV^.versionname<>NIL then writeln(t,'vnam "'+GetPSString(TDV^.versionname),'"');
      if TDV^.comment<>NIL then   writeln(t,'cmmt "'+GetPSString(TDV^.comment),'"');
      if TDV^.protection<>NIL then writeln(t,'prot "'+GetPSString(TDV^.protection),'"');
      if TDV^.cracker<>NIL then   writeln(t,'crck "'+GetPSString(TDV^.cracker),'"');
      if TDV^.releaser<>NIL then  writeln(t,'rels "'+GetPSString(TDV^.cracker),'"');
      if TDV^.fix<>0 then         writeln(t,'fixd '+hexa(TDV^.fix,2));
      if TDV^.fixer<>NIL then     writeln(t,'fixr "'+GetPSString(TDV^.fixer),'"');
      if TDV^.train<>0 then       writeln(t,'trnd '+hexa(TDV^.train,4));
      if TDV^.trainer<>NIL then   writeln(t,'trnr "'+GetPSString(TDV^.trainer),'"');
      if TDV^.modification<>NIL then writeln(t,'mdfd "'+GetPSString(TDV^.modification),'"');
      if TDV^.modificator<>NIL then writeln(t,'mdfr "'+GetPSString(TDV^.modificator),'"');
      writeln(t,'gchp ',hexa(TDV^.graphicschip,2));
      if TDV^.memc<>0 then      writeln(t,'memc ',TDV^.memc);
      if TDV^.mems<>0 then      writeln(t,'mems ',TDV^.mems);
      if TDV^.memf<>0 then      writeln(t,'memf ',TDV^.memf);
      if TDV^.lang<>1 then      writeln(t,'lang ',hexa(TDV^.lang,8));
      write(t,'dord ');
      m:=0; for n:=$00 to $ff do if TDV^.disks[n]<>0 then inc(m);
      if m>0 then begin
        for n:=$00 to $ff do if TDV^.disks[n]<>0 then write(t,hexa(TDV^.disks[n],2));
      end else write(t,'00');
      writeln(t);
      write(t,'comp ');
      write(t,hexa(TDV^.compat.cpu,4));
      write(t,hexa(TDV^.compat.chipset,2));
      write(t,hexa(TDV^.compat.floppy,4));
      write(t,hexa(TDV^.compat.graphics,2));
      write(t,hexa(TDV^.compat.memory,2));
      write(t,hexa(TDV^.compat.memchip,4));
      write(t,hexa(TDV^.compat.memslow,4));
      write(t,hexa(TDV^.compat.memfast,4));
      write(t,hexa(TDV^.compat.kickstart,4));
      write(t,hexa(TDV^.compat.special,4));
      write(t,hexa(TDV^.compat.winuae,8));
      writeln(t,hexa(TDV^.compat.winfellow,8));
      TDV:=TDV^.next;
      writeln(t,')');
    end;

   { disks }
    TDD:=TDS^.disks;
    while TDD<>NIL do begin
      writeln(t,'disk ( ');
      if TDD^.namematch=0 then if TDD^.diskname<>NIL then writeln(t,'dnam "'+GetPSString(TDD^.diskname),'"');
      if TDD^.namematch=1 then if TDD^.diskname<>NIL then writeln(t,'dnmm "'+GetPSString(TDD^.diskname),'"');
      if TDD^.comment<>NIL then  writeln(t,'cmmt "'+GetPSString(TDD^.comment),'"');
     { files }
      TDF:=TDD^.files;
      while TDF<>NIL do begin
        write(t,'file ( ');
        if TDF^.filename<>NIL then    write(t,'fnam "'+GetPSString(TDF^.filename)+'" ');
        if TDF^.description<>NIL then write(t,'desc "'+GetPSString(TDF^.description)+'" ');
        write(t,'size ',TDF^.size,' ');
        write(t,'crcv ',hexa(TDF^.crc32,8),' ');
        write(t,'info ',hexa(TDF^.info,8),' ');
        writeln(t,')');
        TDF:=TDF^.next;
      end;

     { ranges }
      TDR:=TDD^.ranges;
      while TDR<>NIL do begin
        write(t,'range ( ');
        if TDR^.description<>NIL then write(t,'desc "'+GetPSString(TDR^.description)+'" ');
        write(t,'rbeg ',hexa(TDR^.addr1,8),' ');
        write(t,'rend ',hexa(TDR^.addr2,8),' ');
        write(t,'crcv ',hexa(TDR^.crc32,8),' ');
        write(t,'info ',hexa(TDR^.info,8),' ');
        writeln(t,')');
        TDR:=TDR^.next;
      end;

      TDD:=TDD^.next;
      writeln(t,')');
    end;

    TDS:=TDS^.next;
    writeln(t,')');
  end;

  close(t);
end;

var nexts:string;
    nextt:text;

function NextLine:string;
var s:string;
begin
  readln(nextt,s);
  NextLine:=s;
end;

function NextWord:string;
var b1,b2,b3:byte;
begin
  b1:=1;
  while (b1<=length(nexts)) and (nexts[b1]<=#$20) do inc(b1);
  if b1<=length(nexts) then begin
    if nexts[b1]='"' then begin
      b2:=b1+1;
      while (b2<=length(nexts)) and (nexts[b2]<>'"') do inc(b2);
      inc(b1);
    end else begin
      b2:=b1;
      while (b2<=length(nexts)) and (nexts[b2]>#$20) do inc(b2);
    end;
  end else b2:=b1;
  if b2=b1 then begin
    if not eof(nextt) then begin
      nexts:=NextLine;
      NextWord:=NextWord;
    end else NextWord:='';
  end else begin
    NextWord:=copy(nexts,b1,b2-b1);
    nexts:=copy(nexts,b2+1,length(nexts)-b2+1);
  end;
end;

procedure DIDLoad(var DBB:DB_Base; fn:string);
var TDB:DB_Base;
    TDS:DB_Soft;
    TDV:DB_Version;
    TDD:DB_Disk;
    TDF:DB_File;
    TDR:DB_Range;
    n:longword;
    s:string;
    level:byte;
    parent:array[0..2] of string;
begin
  assign(nextt,fn);
  reset(nextt);
  nexts:='';
  new(DBB);
  fillchar(DBB^,sizeof(DBB^),0);
  TDB:=DBB;

  s:=nextword;
  level:=0;
  while s<>'' do begin
   { level switch }
    if s='(' then inc(level);
    if s=')' then dec(level);
   { records declaration }
    if (level=0) and (s='info') then parent[level]:=s else
    if (level=0) and (s='soft') then begin
      parent[level]:=s;
      if TDB^.softs=NIL then begin New(TDB^.softs); TDS:=TDB^.softs; end else begin New(TDS^.next); TDS:=TDS^.next; end;
      fillchar(TDS^,sizeof(TDS^),0);
    end else
    if (level=1) and (s='version') then begin
      parent[level]:=s;
      if TDS^.versions=NIL then begin New(TDS^.versions); TDV:=TDS^.versions; end else begin New(TDV^.next); TDV:=TDV^.next; end;
      fillchar(TDV^,sizeof(TDV^),0);
      TDV^.lang:=1;
    end else
    if (level=1) and (s='disk') then begin
      parent[level]:=s;
      if TDS^.disks=NIL then begin New(TDS^.disks); TDD:=TDS^.disks; end else begin New(TDD^.next); TDD:=TDD^.next; end;
      fillchar(TDD^,sizeof(TDD^),0);
    end else
    if (level=2) and (s='file') then begin
      parent[level]:=s;
      if TDD^.files=NIL then begin New(TDD^.files); TDF:=TDD^.files; end else begin New(TDF^.next); TDF:=TDF^.next; end;
      fillchar(TDF^,sizeof(TDF^),0);
    end else
    if (level=2) and (s='range') then begin
      parent[level]:=s;
      if TDD^.ranges=NIL then begin New(TDD^.ranges); TDR:=TDD^.ranges; end else begin New(TDR^.next); TDR:=TDR^.next; end;
      fillchar(TDR^,sizeof(TDR^),0);
    end else
   { records filling: dbase }
    if (level=1) and (parent[level-1]='info') then begin
      if (s='name') then SetPSString(TDB^.name,nextword) else
      if (s='creator') then SetPSString(TDB^.creator,nextword) else
      if (s='date') then begin
        s:=nextword;
        TDB^.releaseyear:=StrToInt(copy(s,1,4));
        TDB^.releasemonth:=StrToInt(copy(s,5,2));
        TDB^.releaseday:=StrToInt(copy(s,7,2));
      end;
    end else
   { records filling: software }
    if (level=1) and (parent[level-1]='soft') then begin
      if (s='name') then SetPSString(TDS^.gamename,nextword) else
      if (s='rdat') then begin
        s:=nextword;
        TDS^.releaseyear:=StrToInt(copy(s,1,4));
        TDS^.releasemonth:=StrToInt(copy(s,5,2));
        TDS^.releaseday:=StrToInt(copy(s,7,2));
      end else
      if (s='publ') then SetPSString(TDS^.publisher,nextword) else
      if (s='devl') then SetPSString(TDS^.developer,nextword) else
      if (s='code') then SetPSString(TDS^.programm,nextword) else
      if (s='grfx') then SetPSString(TDS^.graphics,nextword) else
      if (s='musi') then SetPSString(TDS^.music,nextword) else
      if (s='sdfx') then SetPSString(TDS^.soundfx,nextword) else
      if (s='dsgn') then SetPSString(TDS^.design,nextword) else
      if (s='cmmt') then SetPSString(TDS^.comment,nextword);
    end else
   { records filling: version }
    if (level=2) and (parent[level-1]='version') then begin
      if (s='gnam') then SetPSString(TDV^.gamename,nextword) else
      if (s='vnam') then SetPSString(TDV^.versionname,nextword) else
      if (s='rdat') then begin
        s:=nextword;
        TDV^.releaseyear:=StrToInt(copy(s,1,4));
        TDV^.releasemonth:=StrToInt(copy(s,5,2));
        TDV^.releaseday:=StrToInt(copy(s,7,2));
      end else
      if (s='publ') then SetPSString(TDV^.publisher,nextword) else
      if (s='devl') then SetPSString(TDV^.developer,nextword) else
      if (s='code') then SetPSString(TDV^.programm,nextword) else
      if (s='grfx') then SetPSString(TDV^.graphics,nextword) else
      if (s='musi') then SetPSString(TDV^.music,nextword) else
      if (s='sdfx') then SetPSString(TDV^.soundfx,nextword) else
      if (s='dsgn') then SetPSString(TDV^.design,nextword) else
      if (s='cmmt') then SetPSString(TDV^.comment,nextword) else

      if (s='prot') then SetPSString(TDV^.protection,nextword) else
      if (s='rels') then SetPSString(TDV^.releaser,nextword) else
      if (s='crck') then SetPSString(TDV^.cracker,nextword) else
      if (s='fixd') then TDV^.fix:=deci(nextword) else
      if (s='fixr') then SetPSString(TDV^.fixer,nextword) else
      if (s='trnd') then TDV^.train:=deci(nextword) else
      if (s='trnr') then SetPSString(TDV^.trainer,nextword) else
      if (s='mdfd') then SetPSString(TDV^.modification,nextword) else
      if (s='mdfr') then SetPSString(TDV^.modificator,nextword) else
      if (s='gchp') then TDV^.graphicschip:=Deci(nextword) else
      if (s='memc') then TDV^.memc:=StrToInt(nextword) else
      if (s='mems') then TDV^.mems:=StrToInt(nextword) else
      if (s='memf') then TDV^.memf:=StrToInt(nextword) else
      if (s='lang') then TDV^.lang:=Deci(nextword) else
      if (s='dord') then begin
        s:=nextword; n:=0;
        while s<>'' do begin
          TDV^.disks[n]:=Deci(copy(s,1,2));
          s:=copy(s,3,length(s)-2);
          inc(n);
        end;
      end else
      if (s='comp') then begin
        s:=nextword;
        TDV^.compat.cpu       :=Deci(copy(s,01,4));
        TDV^.compat.chipset   :=Deci(copy(s,05,2));
        TDV^.compat.floppy    :=Deci(copy(s,07,4));
        TDV^.compat.graphics  :=Deci(copy(s,11,2));
        TDV^.compat.memory    :=Deci(copy(s,13,2));
        TDV^.compat.memchip   :=Deci(copy(s,15,4));
        TDV^.compat.memslow   :=Deci(copy(s,19,4));
        TDV^.compat.memfast   :=Deci(copy(s,23,4));
        TDV^.compat.kickstart :=Deci(copy(s,27,4));
        TDV^.compat.special   :=Deci(copy(s,31,4));
        TDV^.compat.winuae    :=Deci(copy(s,35,8));
        TDV^.compat.winfellow :=Deci(copy(s,43,8));
      end;
    end else
   { records filling: disk }
    if (level=2) and (parent[level-1]='disk') then begin
      if (s='dnam') then begin SetPSString(TDD^.diskname,nextword); TDD^.namematch:=0; end else
      if (s='dnmm') then begin SetPSString(TDD^.diskname,nextword); TDD^.namematch:=1; end else
      if (s='cmmt') then SetPSString(TDD^.comment,nextword);
    end else
   { records filling: disk }
    if (level=3) and (parent[level-1]='file') then begin
      if (s='fnam') then SetPSString(TDF^.filename,nextword) else
      if (s='desc') then SetPSString(TDF^.description,nextword) else
      if (s='size') then TDF^.size:=StrToInt(nextword) else
      if (s='info') then TDF^.info:=Deci(nextword) else
      if (s='crcv') then TDF^.crc32:=Deci(nextword);
    end else
   { records filling: range }
    if (level=3) and (parent[level-1]='range') then begin
      if (s='desc') then SetPSString(TDR^.description,nextword) else
      if (s='rbeg') then TDR^.addr1:=Deci(nextword) else
      if (s='rend') then TDR^.addr2:=Deci(nextword) else
      if (s='info') then TDR^.info:=Deci(nextword) else
      if (s='crcv') then TDR^.crc32:=Deci(nextword);
    end;
    s:=nextword;
  end;
  close(nextt);
end;

end.
